# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import gtk
import time
import pango
import gobject

# for drag and drop
import os
import urllib

import UserList
import SmilieWindow
import FontStyleWindow
from Theme import resizePixbuf
from Parser import PangoDataType

import emesenelib.common
import emesenelib.ContactData

import Widgets

from htmltextview import HtmlTextView

class ConversationUI(gtk.VBox):
    '''this class represent all the widgets that are inside a tab
    also hold the tab widget because there is no better place than
    this to hold it...'''
    
    def __init__(self, controller, parentConversation):
        gtk.VBox.__init__(self, spacing=3)
        self.set_border_width(0)
        
        self.parentConversation = parentConversation
        self.controller = controller
        self.config = self.controller.config
        
        # the header that contain the nick, the PM, status etc
        self.header = Header(self, controller)
        # the widget that contain the tabs and the conversations
        self.tabWidget = TabWidget(self, controller)
        self.tabWidget.show()
        # the widget where you can put the text
        self.input = InputWidget(self, controller, \
            parentConversation.isCurrent)
        self.input.show()
        # the status bar
        self.status = gtk.Statusbar()

        self.toolbarinput = gtk.HBox()
        self.toolbarinput.show()
        # the list of users when we are on a group chat
        self.listOfUsers = UserList.UserList(self.controller, \
            self.controller.theme, self.config, False)
        
        # the scroll for the list of users
        self.scrollList = gtk.ScrolledWindow()
        self.scrollList.set_shadow_type(gtk.SHADOW_IN)
        self.scrollList.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        self.scrollList.set_size_request(111, 0)
        self.scrollList.add(self.listOfUsers)

        # the scroll for the textview below
        self.scroll = gtk.ScrolledWindow()
        self.scroll.set_shadow_type(gtk.SHADOW_IN)
        self.scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        
        # the widget where the conversation is shown
        self.textview = HtmlTextView(controller, \
            self.parentConversation.textBuffer, self.scroll)
        self.textview.set_wrap_mode(gtk.WRAP_WORD_CHAR)
        self.textview.set_left_margin(6)
        self.textview.set_right_margin(6)
        self.textview.set_editable(False)
        self.textview.set_cursor_visible(False)
        self.textview.connect('key-press-event', self.onTextviewKeyPress)

        self.scroll.add(self.textview)
        self.scroll.show_all()

        # Friends avatar
        
        #self.imageFrame = gtk.AspectFrame()
        self.image = Widgets.avatarHolder(cellKeyPosition = gtk.ANCHOR_NORTH)

        contact = self.controller.getContact(\
            self.parentConversation.switchboard.firstUser)
        if contact:
            self.image.set_from_pixbuf(\
                self.controller.theme.getUserDisplayPicture(contact))
        #self.imageFrame.add(self.image)

        # TODO: references
        self.controller.msn.connect('display-picture-changed', self.displayPictureChanged)
        self.controller.msn.connect('user-attr-changed', self.onUserAttrChanged)
        
        self.hidebutton = gtk.Button()
        self.hidebutton.connect('clicked', self.hideShowButton)
        self.hidebutton.set_relief(gtk.RELIEF_NONE)
        self.hidebuttonArrow = Widgets.TinyArrow(gtk.ARROW_RIGHT)
        self.hidebutton.add(self.hidebuttonArrow)
        self.hidebutton.show_all()
        
        # Vbox to pack both the userlist and friend display image
        self.vbox = gtk.VBox()
        self.vbox.pack_start(self.image, False,False)
        self.vbox.pack_start(self.scrollList, True, True)
        self.vbox.show()

        self.filler = gtk.VBox()
        self.vbox2 = gtk.VBox()
        self.vbox2.pack_start(self.hidebutton, False, False)
        self.vbox2.pack_start(self.filler, True, True)
        self.vbox2.show_all()

        # HBox to pack the VBox above and the Hide/Show Friends Avatar button
        hbox1 = gtk.HBox()
        hbox1.pack_start(self.vbox, True, True)
        hbox1.pack_start(self.vbox2, False)
        hbox1.show()
        
        hbox2 = gtk.HBox(spacing=2)
        hbox2.set_border_width(2)
        hbox2.pack_start(self.scroll, True, True)
        hbox2.pack_start(hbox1, False, False)
        hbox2.show()

        # the hbox that pack the textview, the images of the other users
        # and the userlist
        hbox3 = gtk.HBox()
        hbox3.pack_start(hbox2, True, True)
        hbox3.show()
        
        self.toolbarinput.pack_start(self.input, True, True)
        self.toolbarinput.connect('size-allocate', self.onToolbarinputResize)

        # the vertical panel that divide the textview and the input 
        # (toolbar and input)
        vpaned = gtk.VPaned()
        vpaned.pack1(hbox3, True, True)
        vpaned.pack2(self.toolbarinput, False)
        vpaned.show()
        
        self.transfers = FtBarWidget(self.controller, self, 
                                        parentConversation)
        self.transfers.set_no_show_all(True)
                
        self.pack_start(self.header, False, False)
        self.pack_start(vpaned, True, True)
        self.pack_start(self.transfers, False, False)
        self.pack_start(self.status, False, False)
      
        self.messageWaiting = {}
        self.contactTyping = {}

        self.typingTimeoutID = 0
        self.closed = False

        self.last_mark = None

        self.show()
        self.update()
    
    # TODO: make it work again
    def onToolbarinputResize(self, *args):
        alloc = self.toolbarinput.get_allocation()
        self.config.user['convInputHeight'] = alloc.height
        self.scrollToBottom()
    
    def onTextviewKeyPress(self, widget, event):
        # don't focus inputbox if Control is pressed (Ctrl-C etc.)
        if event.keyval == gtk.keysyms.Control_L or \
                event.keyval == gtk.keysyms.Control_R or \
                event.state & gtk.gdk.CONTROL_MASK:
            return

        self.input.input.emit('key-press-event', event)
        self.input.grabFocus()
    
    def close(self):
        self.closed = True
    
    def setInputEnabled(self, enable):
        self.input.input.set_sensitive(enable)
    
    def onUserAttrChanged(self, msn, contact):
        if contact.email in self.parentConversation.getMembers():
            self.update()
    
    def noneIsTyping(self):
        try:
            for contact in self.email_list:
                if self.contactTyping[contact] == True:
                    return False
            return True
        except:
            return False
            
    def update(self):
        self.header.update()
        self.tabWidget.update()
        self.input.update()
        self.email_list = [mail.email for mail in self.parentConversation.getMembersDict().values()]
        
        # add new user in conversation
        for email in self.email_list:
            if email not in self.contactTyping.keys():
                self.contactTyping[email] = False
            if email not in self.messageWaiting.keys():
                self.messageWaiting[email] = False
        
        for email in self.contactTyping.keys():
            if self.contactTyping[email]:
                self.setTyping(email)
            else:
                self.setDefault(email)

        isGroupChat = len(self.parentConversation.getMembers()) > 1        
        if isGroupChat:
            members = self.parentConversation.getMembersDict()
            d = {'members' : emesenelib.ContactData.Group('members')}
            for member in members.values():
                d['members'].setUser(member.email, member)

            self.listOfUsers.fill(d)

        # if you want to show the avatar and hide the grouplist or
        # viceversa first do the hide and then the show, because this methods
        # also hide the container vbox to not let an empty widget there
        if isGroupChat:
            self.hideAvatar()
            self.hidebutton.hide()
            self.showUserList()
        elif self.config.user['showAvatars']:
            self.updateAvatarOther()
            self.hidebutton.show()
            self.hideUserList()
            if self.config.user['showAvatarOther']:
                self.hidebuttonArrow.set(gtk.ARROW_RIGHT, gtk.SHADOW_NONE)
                self.showAvatar()
            else:
                self.hidebuttonArrow.set(gtk.ARROW_LEFT, gtk.SHADOW_NONE)
                self.hideAvatar()
        else:
            self.hideAvatar()
            self.hideUserList()
            self.hidebutton.hide()
            
        if not self.config.user['showHeader']:
            self.header.hide()
        else:
            self.header.show()

        if not self.config.user['showTabCloseButton']:
            self.tabWidget.closeButton.hide()
        else:
            self.tabWidget.closeButton.show()

        if not self.config.user['showStatusBar']:
            self.status.hide()
        else:
            self.status.show()
        
        inputHeight = self.config.user['convInputHeight']
        self.toolbarinput.set_size_request(0, inputHeight)

    def scrollToBottom(self):
        '''scroll to the end of the conversation'''
        self.textview.scrollToBottom()
        
    def showUserList(self):
        self.vbox.show()
        self.scrollList.show_all()
        self.listOfUsers.show()
        
    def hideUserList(self):
        self.vbox.hide()
        self.scrollList.hide()
        self.listOfUsers.hide()

    def showAvatar(self):
        self.vbox.show()
        self.image.show_all()

    def hideAvatar(self):
        self.vbox.hide()
        self.image.hide()
        
    def hideShowButton(self, *args):
        if self.hidebuttonArrow.arrow_type == gtk.ARROW_LEFT:
            self.hidebuttonArrow.set(gtk.ARROW_RIGHT, gtk.SHADOW_NONE)
            if len(self.parentConversation.getMembers()) > 1:
                self.showUserList()
            else:
                self.showAvatar()
                self.config.user['showAvatarOther'] = True
        else:
            self.hidebuttonArrow.set(gtk.ARROW_LEFT, gtk.SHADOW_NONE)
            if len(self.parentConversation.getMembers()) > 1:
                self.hideUserList()
            else:
                self.hideAvatar()
                self.config.user['showAvatarOther'] = False
    
    def rebuildStatusText(self):
        '''Builds the text displayed in the statusbar, based in
        self.contactTyping. The "output" is a comma separated list
        of (full) mails and "is/are typing a message..." '''

        mails = [x for x in self.contactTyping.keys() \
                  if self.contactTyping[x] == True]

        string = ''

        if len(mails) > 0:
            comma = ', ' # TODO: gettext?
            for mail in mails:
                if self.config.user['showMailTyping']:
                    string += str(mail)
                else:
                    contact = self.controller.getContact(mail)
                    if contact:
                        string += contact.nick
                    else:
                        string += str(mail)
                string += comma
                
            # yes, i know we don't need to deal with utf-8 here, but...
            string = str(unicode(string)[:-len(comma)])
            
            if len(mails) == 1:
                string += ' ' + _('is typing a message...')
            else:
                string += ' ' + _('are typing a message...')

        #this is a hack to skip the statusbar stack
        self.status.get_children()[0].get_children()[0].set_text(string)

    def setMessageWaiting(self, mail):
        if self.parentConversation.getIsCurrent():
            return self.setDefault(mail)

        self.tabWidget.setMessageWaiting()
        self.header.setDefault()
        if mail:
            self.contactTyping[mail] = False
            self.messageWaiting[mail] = True
            self.rebuildStatusText()
        
    def setTyping(self, mail):
        if self.messageWaiting.has_key(mail) and self.messageWaiting[mail]: return
        self.header.setTyping()
        self.tabWidget.setTyping()
        
        self.contactTyping[mail] = True
        self.rebuildStatusText()

        if self.typingTimeoutID > 0:
            gobject.source_remove(self.typingTimeoutID)

        self.typingTimeoutID = gobject.timeout_add(8000, \
            self.clearTyping, mail)

    def clearTyping(self, mail):
        if mail in self.messageWaiting and self.messageWaiting[mail]:
            self.setMessageWaiting(mail)
        else:
            self.setDefault(mail)
        self.contactTyping[mail] = False
        return False

    def setDefault(self, mail):
        #if self.noneIsTyping(): 
        self.tabWidget.setDefault()
        self.header.setDefault()
        if mail:
            self.contactTyping[mail] = False
            self.messageWaiting[mail] = False
            self.rebuildStatusText()

    def displayPictureChanged(self, msnp, switchboard, msnobj, email):
        if self.parentConversation.getSwitchboard().firstUser == email:
            self.updateAvatarOther()
    
    def updateAvatarOther(self):
        '''Update avatar of the other user'''
        members = self.parentConversation.getMembers()
        if len(members) == 1:
            contact = self.controller.getContact(members[0])
            if contact:
                self.image.set_from_pixbuf(
                    self.controller.theme.getUserDisplayPicture(contact))
    
    # TODO: getters
    def getTextView(self):
        return self.textview
    
    def getParentConversation(self):
        return self.parentConversation
                
class Header(gtk.HBox):
    '''the header of the conversation'''
    
    def __init__(self, parentUI, controller):
        '''Contructor'''
        gtk.HBox.__init__(self)
        self.parentUI = parentUI
        self.controller = controller
        self.config = controller.config
        self.set_border_width(2)
        self.set_spacing(2)
        self.parser = controller.unifiedParser
        
        self.statusImage = gtk.Image()
        self.statusImage.set_from_pixbuf(self.controller.theme.getImage('groupChat'))
        
        self.label = gtk.Label()
        
        self.label.set_selectable(True)
        self.label.unset_flags(gtk.CAN_FOCUS)
        
        self.label.set_alignment(0.0, 0.5)
        self.label.set_line_wrap(True)
        self.label.set_ellipsize(pango.ELLIPSIZE_END)
                
        self.label.set_markup('<b><i>' + _('Connecting...') + '</i></b>')
        
        self.reconnect = gtk.Button(_('Reconnect'))
        self.reconnect.connect('clicked', self.reconnectClicked)
        self.pack_start(self.statusImage, False, False)
        self.pack_start(self.label)
        self.pack_start(self.reconnect, False, False)
        self.statusImage.show()
        self.label.show()
        self.show()
        
    def reconnectClicked(self, button):
        self.parentUI.parentConversation.reconnect()
        self.reconnect.hide()
    
    def update(self):
        conversation = self.parentUI.parentConversation
        theme = self.controller.theme
        members = conversation.getMembersDict()
        if self.config.user['showTabImageStatus']:
            self.statusImage.hide()
            return
        if conversation.switchboard.status == 'error':
            self.setLabel()
            self.parentUI.parentConversation.parentConversationWindow.menu.update()
        self.statusImage.set_from_pixbuf(conversation.getStatusIcon())

    def setTyping(self):
        typingColor = self.config.user['typingColor']
        styleOpen = "<b><span foreground='" + typingColor + "'>"
        styleClose = "</span></b>"
        self.setLabel(styleOpen, styleClose)
    
    def setLabel(self, styleOpen='<b>', styleClose='</b>'):
        '''Sets the header label, the one that often includes bold nick,
        gray psm and mail. It also checks the switchboard status.'''
        conversation = self.parentUI.parentConversation
        members = conversation.getMembersDict()
        personalMessageColor = self.config.user['personalMessageColor']
        switchboard = conversation.switchboard

        if switchboard.status == 'error':
            if switchboard.error == 215:
                desc = _('The user is already present')
            elif switchboard.error == 217:
                desc = _('The user is offline')
            else:
                desc = str(switchboard.error)
            
            self.text = '%s<i> %s (%s) </i>%s' % \
                (styleOpen, _('Can\'t connect'), desc, styleClose)
            self.label.set_markup(self.text)
            self.reconnect.show()
            return
        self.reconnect.hide()

        if len(members) == 1:
            contact = members.values()[0]
            parts = self.parser.getParser(contact.nick, PangoDataType).get()
            nick = ''
            for part in parts:
                nick += str(part)

            self.text = styleOpen + nick + \
                styleClose + "<span foreground='%s'>" % personalMessageColor
            
            if contact.personalMessage:
                self.text += '\n' + \
                    emesenelib.common.escape(contact.personalMessage)
        
            self.text += "\n%s</span>" % contact.email
        elif len(members) > 1:
            self.text = styleOpen + _('Group chat') + styleClose + \
                "\n<span foreground='%s'>" % personalMessageColor + \
                str(len(members)) + ' ' + _('Members') + "</span>"
            
            self.statusImage.set_from_pixbuf(
                self.controller.theme.getImage('groupChat') )
        else:
            self.text = styleOpen + '<i>' + _('Connecting...') + '</i>' + styleClose
                
        self.label.set_markup(self.text)
          
    def setDefault(self):
        self.setLabel()
        
class TabWidget(gtk.HBox):
    '''this class represent the widget that is inside the tab
    it contains a label and a close button'''
    
    def __init__(self, parentUI, controller):
        '''Contructor'''
        gtk.HBox.__init__(self)

        self.parentUI = parentUI
        self.controller = controller
        self.config = controller.config
        
        self.text = ''
        self.markup = ''
        self.updateText()

        self.label = gtk.Label()
        self.label.set_use_markup(True)
        self.label.set_markup(self.markup)
        self.label.set_ellipsize(pango.ELLIPSIZE_END)
        
        img = gtk.Image()
        img.set_from_stock(gtk.STOCK_CLOSE, gtk.ICON_SIZE_MENU)
        
        self.closeButton = gtk.Button()
        self.closeButton.set_image(img)
        self.closeButton.set_relief(gtk.RELIEF_NONE)
        
        self.statusImage = gtk.Image()
        
        self.pack_start(self.statusImage, False, False)
        self.pack_start(self.label)
        self.pack_start(self.closeButton, False, False)

        self.closeButton.connect('clicked', self.onCloseClicked)
        self.show_all()
    
    def onCloseClicked(self, button):
        self.parentUI.parentConversation.close()
    
    def update(self):
        self.updateText()
        #if not self.config.user['showTabImageStatus']:
        #    return
        members = self.parentUI.parentConversation.getMembersDict()
        
        if len(members) == 1:
            contact = members.values()[0]
            basepixbuf = self.controller.theme.statusToPixbuf(contact.status)
        else:
            basepixbuf = self.controller.theme.getImage('userPanel')

        self.statusImage.set_from_pixbuf(resizePixbuf(basepixbuf, 16, 16))

    def updateText(self):
        title = self.parentUI.parentConversation.getTitle()
        parts = self.controller.unifiedParser.getParser(title,
            PangoDataType).get()
        self.markup = ''.join([str(x) for x in parts])
        self.text = self.controller.unifiedParser.getParser(title).get()

    def setDefault(self):
        self.label.set_markup(self.markup)
        
    def setTyping(self):
        typingColor = self.config.user['typingColor']
        self.label.set_markup('<span foreground="%s">%s</span>' % (typingColor, self.text))
    
    def setMessageWaiting(self):
        messageWaitingColor = self.config.user['messageWaitingColor']
        self.label.set_markup('<span foreground="%s">%s</span>' % (messageWaitingColor, self.text))
    
class LocalAvatar(gtk.HBox):
    def __init__(self, parentUI, controller):
        gtk.HBox.__init__(self)
        
        self.parentUI = parentUI
        self.controller = controller
        self.config = controller.config
        
        self.hidebutton = gtk.Button()
        self.hidebutton.connect('clicked', self.hideShowButton)    
        self.hidebutton.set_relief(gtk.RELIEF_NONE)
        self.hidebuttonArrow = Widgets.TinyArrow(gtk.ARROW_RIGHT)
        self.hidebutton.add(self.hidebuttonArrow)
        
        self.localimage = Widgets.avatarHolder(cellKeyPosition = gtk.ANCHOR_SOUTH)
        self.imageEventBox = gtk.EventBox()
        self.imageEventBox.set_events(gtk.gdk.BUTTON_PRESS_MASK)
        self.imageEventBox.set_visible_window(False)
        self.imageEventBox.connect('button-press-event', self.changeAvatar)
        self.imageEventBox.add(self.localimage)
        
        if self.controller.avatar != None:
            self.localimage.set_from_pixbuf(self.controller.avatar.getImage())
        else:
            self.pixbuf = self.controller.theme.getImage('icon96')
            self.localimage.set_from_pixbuf(self.pixbuf)
        
        self.controller.connect('avatar-changed', self.localDisplayPictureChanged)
        
        self.filler = gtk.VBox()
        
        self.vboxButtonFrame = gtk.VBox()
        self.vboxButtonFrame.pack_start(self.hidebutton, False, False)
        self.vboxButtonFrame.pack_start(self.filler, True, True)
        
        self.filler2 = gtk.VBox()
        
        self.vboxAvatarFrame = gtk.VBox()
        self.vboxAvatarFrame.pack_start(self.imageEventBox, False, False)
        self.vboxAvatarFrame.pack_start(self.filler2, True, True)
        
        self.pack_start(self.vboxAvatarFrame, False, False)
        self.pack_start(self.vboxButtonFrame, False, False)
        self.update()
        
    def update(self):
        if self.config.user['showAvatars']:
            self.hidebutton.show()
            if not self.config.user['showAvatarMine']:
                self.imageEventBox.hide()
                self.hidebuttonArrow.set(gtk.ARROW_LEFT, gtk.SHADOW_NONE)
            else:
                self.imageEventBox.show()
                self.hidebuttonArrow.set(gtk.ARROW_RIGHT, gtk.SHADOW_NONE)
        else:
            self.imageEventBox.hide()
            self.hidebutton.hide()
        
    def displayPictureChanged(self, msnp, email):
        if self.parentUI.parentConversation.getSwitchboard().firstUser == email:
            #self.image.set_from_pixbuf(self.controller.theme.getUserDisplayPicture(email))  
            print 'check here because self.image soesnt exist (ConversationUI 720)'

    def localDisplayPictureChanged(self, controller):
        self.localimage.set_from_pixbuf(self.controller.avatar.getImage()) 

    def changeAvatar(self, widget, event):
        self.controller.set_picture_dialog()
            
    def hideShowButton(self, *args):
        if self.config.user['showAvatars']:
            self.hidebutton.show()
            if self.config.user['showAvatarMine']:
                self.hidebuttonArrow.set(gtk.ARROW_LEFT, gtk.SHADOW_NONE)
                self.imageEventBox.hide()
                self.config.user['showAvatarMine'] = False
            else:
                self.hidebuttonArrow.set(gtk.ARROW_RIGHT, gtk.SHADOW_NONE)
                self.imageEventBox.show()
                self.config.user['showAvatarMine'] = True
        else:
            self.imageEventBox.hide()
            self.hidebutton.hide()

class InputWidget(gtk.HBox):
    '''This class represents the input widgets (text entry) and avatar (if enabled)'''
    
    TARGET_TYPE_URI_LIST = 80
    DND_LIST = [('text/uri-list', 0, TARGET_TYPE_URI_LIST)]
    
    def __init__(self, parentUI, controller, setFocus=True):
        gtk.HBox.__init__(self)
        self.set_border_width(2)
        self.parentUI = parentUI
        self.controller = controller
        self.config = controller.config
        self.lastKeyPressed = 0
        self.history = []
        self.historyIndex = -1
        #self.set_border_width(2)

        # the local avatar
        self.localAvatar = LocalAvatar(self, controller)
        # the toolbar
        self.toolbar = ToolbarWidget(self.parentUI, controller)
        self.toolbar.set_style(gtk.TOOLBAR_ICONS)
        self.toolbar.set_small(self.config.user['smallIcons'])

        self.input = Widgets.inputBox()
        self.input.set_left_margin(6)
        self.input.set_right_margin(6)
        self.input.set_wrap_mode(gtk.WRAP_WORD_CHAR)
        
        #self.testbutton = gtk.Button("Senden")
        #self.input.add_child_in_window(
        #    self.testbutton, gtk.TEXT_WINDOW_TEXT, 500, 50)
        #self.testbutton.show()
        
        self.inputBuffer = self.input.get_buffer()
        
        self.scrollInput = gtk.ScrolledWindow()
        self.scrollInput.set_policy(gtk.POLICY_AUTOMATIC, 
                                    gtk.POLICY_AUTOMATIC)
        self.scrollInput.add(self.input)
        
        frameInput = gtk.Frame()
        frameInput.set_shadow_type(gtk.SHADOW_IN)
        
        self.sendAlignment = gtk.Alignment(0.0, 0.5, 0.0, 0.0)
        
        self.sendbutton = gtk.Button(_('Send'))
        self.sendbutton.connect('clicked', self.message_send)
        
        self.sendAlignment.add(self.sendbutton)

        avatar_frame = gtk.AspectFrame(xalign=0.0, yalign=1.0)
        avatar_frame.add(self.localAvatar)
        avatar_frame.set_shadow_type(gtk.SHADOW_NONE)

        self.vbox = gtk.VBox()
        
        self.vbox.pack_start(self.toolbar, False)
        self.vbox.pack_start(self.scrollInput, True, True)
        frameInput.add(self.vbox)
        self.pack_start(frameInput, True, True)
        self.pack_start(self.sendAlignment, False, False)
        self.pack_start(avatar_frame, False, False)

        self.inputBuffer.connect('changed', self.on_changed_event)
        self.input.connect('key-press-event', self.on_key_press_event)

        self.input.connect('drag-data-received', self.on_drag_data_received)
        self.input.drag_dest_set(gtk.DEST_DEFAULT_MOTION | \
            gtk.DEST_DEFAULT_HIGHLIGHT | gtk.DEST_DEFAULT_DROP, \
            InputWidget.DND_LIST, gtk.gdk.ACTION_COPY)
        self.last_dropped_file = ''
        
        self.input.connect_after('message-send', self.message_send)
        self.input.connect_after('escape-pressed', 
                                    self.on_escape_pressed_event)
        
        self.input.connect_after('map-event', self.on_input_map_event)
        
        if setFocus:
            self.input.grab_focus()
        
        self.tag = None
        self.id_timeout = None
        self.parse_off = 0
        self.applyAttrsToInput()
        self.show_all()

    def update(self):
        '''update the inner componnents if needed'''
        self.localAvatar.update()
        self.toolbar.update()
        
        if not self.config.user['showToolbar']:
            self.toolbar.hide()
        else:
            self.toolbar.show()
            
        if not self.config.user['showSendButton']:
            self.sendAlignment.hide()
        else:
            self.sendAlignment.show()

    def transformEmo(self, *args):
        ''' transform smiley shorcuts in pixbuf '''
        if not self.controller or not self.controller.customEmoticons:
            return

        if not self.controller.config.user['parseSmilies']:
            return

        theme = self.controller.theme
        emos = []
        customEmo = self.controller.customEmoticons
        
        for code in theme.getSmileysList():
            iter_start = self.inputBuffer.get_start_iter()
            iter_start.set_offset(self.parse_off)
            while iter_start.forward_search(code, 
                                gtk.TEXT_SEARCH_VISIBLE_ONLY):
                iter_pos, iter_end = iter_start.forward_search(code, \
                    gtk.TEXT_SEARCH_VISIBLE_ONLY)
                pixbuf = theme.getSmiley(code, False)
                pixbuf.shortcut = code
                
                tag = gtk.TextTag()
                tag.set_property('invisible', True)
                tagtable = self.inputBuffer.get_tag_table()
                tagtable.add(tag)
                self.inputBuffer.apply_tag(tag, iter_pos, iter_end)
                
                mark1 = self.inputBuffer.create_mark(None, iter_pos)
                mark2 = self.inputBuffer.create_mark(None, iter_end)
                
                img = gtk.Image()
                img.set_from_pixbuf(pixbuf)
                img.show()
                img.shortcut = code
                
                emos.append((img, mark1, mark2))
                iter_start = iter_end

        for code, filename in customEmo.list.iteritems():
            iter_start = self.inputBuffer.get_start_iter()
            iter_start.set_offset(self.parse_off)
            while iter_start.forward_search(code, 
                                            gtk.TEXT_SEARCH_VISIBLE_ONLY):
                iter_pos,iter_end = iter_start.forward_search(code, \
                    gtk.TEXT_SEARCH_VISIBLE_ONLY)
                pixbuf = gtk.gdk.pixbuf_new_from_file(filename)
                pixbuf = resizePixbuf(pixbuf, 24, 24)
                                
                tag =  gtk.TextTag()
                tag.set_property('invisible', True)
                tagtable = self.inputBuffer.get_tag_table()
                tagtable.add(tag)
                self.inputBuffer.apply_tag(tag, iter_pos, iter_end)
                
                mark1 = self.inputBuffer.create_mark(None, iter_pos)
                mark2 = self.inputBuffer.create_mark(None, iter_end)
                
                img = gtk.Image()
                img.set_from_pixbuf(pixbuf)
                img.show()
                img.shortcut = code
                
                emos.append((img, mark1, mark2))
                iter_start = iter_end
        
        for emo in emos:
            iterStart = self.inputBuffer.get_iter_at_mark(emo[1])
            iterEnd = self.inputBuffer.get_iter_at_mark(emo[2])

            self.inputBuffer.delete(iterStart, iterEnd)
            iterStart = self.inputBuffer.get_iter_at_mark(emo[1])
            #self.inputBuffer.insert_pixbuf(iterStart, emo[0])
            
            anchor = self.inputBuffer.create_child_anchor(iterStart)
                        
            self.input.add_child_at_anchor(emo[0], anchor)
            
        iter_off = self.inputBuffer.get_end_iter()
        if iter_off.inside_word() or iter_off.ends_word():
            iter_off.backward_word_start()
        self.parse_off = iter_off.get_offset()
    
    def grabFocus(self):
        self.input.grab_focus() 
        
    def getBuffer(self):
        return self.inputBuffer
    
    def clearMessageInput(self):
        self.setInputText('')
        return True
    
    def sendMessage(self):
        if self.getInputText() != '':
            # Message size is limited so we must truncate it to send 
            # it in several parts
            message = self.getInputText()
            
            self.parentUI.parentConversation.sendMessage(message)
            self.setInputText('')
            self.history.append(message)
            self.historyIndex = -1
        return True
    
    def setInputText(self , string , tag = None):
        if tag == None:
            self.inputBuffer.set_text(string)
        else:
            pass
        
    def getInputText(self):
        '''return the text in the input textview'''
        iterStart, iterEnd = self.inputBuffer.get_bounds()
        text = self.inputBuffer.get_slice(iterStart, iterEnd)
        emos = []
        #TODO: magic string '\xef\xbf\xbc'
        while iterStart.forward_search('\xef\xbf\xbc', \
            gtk.TEXT_SEARCH_VISIBLE_ONLY):

            iterPos, iterEnd = iterStart.forward_search('\xef\xbf\xbc', \
                gtk.TEXT_SEARCH_VISIBLE_ONLY)
            anchor = iterPos.get_child_anchor()
            if anchor and anchor.get_widgets():
                emos.append(anchor.get_widgets()[0].shortcut)
            elif anchor is None:
                emos.append('')
            #shortcut = iterPos.get_child_anchor().shortcut
            
            iterStart = iterEnd

        for emo in emos:
            text = text.replace('\xef\xbf\xbc', emo, 1)
            
        return text

    def appendInputText(self, text, tag = None):
        '''append the given text to the inputBuffer, if tag != None insert
        with the given tag'''
        self.inputBuffer.insert_at_cursor(text)
        
    def on_changed_event(self, *args):
        '''Method called when text is inserted in the textbuffer (typing)
        First, it checks if the message length is >= 5 or if
        the message is a slash command ("^\/[^\/]" is confusing)
        Then, checks the last key pressed time, and if it's more
        than 5 seconds, send a typing message'''
          
        if self.inputBuffer.get_start_iter().get_offset() < self.parse_off:
            self.parse_off = 0
        if self.id_timeout != None:
            gobject.source_remove(self.id_timeout)
        
        if not self.getInputText().startswith('/'):
            self.id_timeout = gobject.timeout_add(200, self.transformEmo)
        
        text = self.getInputText()
        if len(text) < 5 or (text[0] == '/' and text[1] != '/'):
            return self.updateInputFormat()
        
        # sents user typing notification
        actualTime = int(time.time())
        
        if (actualTime - self.lastKeyPressed) > 5 and \
           self.config.user['sendTyping']:
            self.lastKeyPressed = actualTime

            try:
                self.parentUI.parentConversation.sendIsTyping()
            except Exception , e:
                print str(e)
                
        self.updateInputFormat()
                
    def applyAttrsToInput(self):
        '''apply the current attributes to the text in input'''

        self.tag = self.parentUI.parentConversation.getTextTag()
        self.inputBuffer.get_tag_table().add(self.tag)
        self.tag.set_property('left-margin', 6)
        self.tag.set_priority(self.inputBuffer.get_tag_table().get_size() - 1)
        self.inputBuffer.apply_tag(self.tag,
                                            self.inputBuffer.get_start_iter(),
                                            self.inputBuffer.get_end_iter())
        
        self.controller.emit('input-format-changed', self.input)
        
    def updateInputFormat(self):
        if self.tag != None:
            self.inputBuffer.apply_tag(self.tag,
                                        self.inputBuffer.get_start_iter(),
                                        self.inputBuffer.get_end_iter())
        
    def message_send(self, widget):
        if self.id_timeout != None:
            gobject.source_remove(self.id_timeout)
            self.id_timeout = None

        return self.sendMessage()

    def on_input_map_event(self, widget, event):
        if self.parentUI.getParentConversation().getIsCurrent():
            self.input.grab_focus() 

    def on_escape_pressed_event(self, widget):
        if not self.parentUI.closed and not self.config.user['disableEsc']:
            self.parentUI.parentConversation.close()

    def on_key_press_event(self , widget, event):
        if event.keyval == gtk.keysyms.space:
            if self.id_timeout != None:
                gobject.source_remove(self.id_timeout)
                self.id_timeout = None
            if not self.getInputText().startswith('/'):
                self.transformEmo()

        elif event.keyval in (gtk.keysyms.Up, gtk.keysyms.Down) and \
            (event.state & gtk.gdk.CONTROL_MASK):
            up = event.keyval == gtk.keysyms.Up
            
            if up and self.historyIndex == -1 and len(self.history) > 0:
                self.historyIndex = len(self.history) - 1
                if self.getInputText() != '':
                    self.history.append(self.getInputText())
            
            elif self.historyIndex > 0 and self.historyIndex < len(self.history):
                if up: self.historyIndex -= 1
                else: self.historyIndex += 1
            
            if self.historyIndex > -1 and self.historyIndex < len(self.history):
                self.setInputText(self.history[self.historyIndex])
            elif self.historyIndex != -1:
                self.setInputText('')

    def on_drag_data_received(self, widget, context, x, y, selection,
                              target_type, timestamp):
        '''Callback to on_drag_data_received'''
        
        # Based on:
        # http://faq.pygtk.org/index.py?req=show&file=faq23.031.htp
        
        print "Dropped", repr(selection.data)

        if target_type == InputWidget.TARGET_TYPE_URI_LIST and \
           selection.data != self.last_dropped_file:
            
            uri = selection.data.strip()
            uri_splitted = uri.split()
            for uri in uri_splitted:
                path = self.get_file_path_from_dnd_dropped_uri(uri)
                if os.path.isfile(path):
                    self.parentUI.parentConversation.sendFile(path)
            self.last_dropped_file = str(selection.data)

    def get_file_path_from_dnd_dropped_uri(self, uri):
        '''Parses an URI received from dnd and return the real path'''

        path = urllib.url2pathname(uri) # escape special chars
        path = path.strip('\r\n\x00') # remove \r\n and NULL

        # get the path to file
        if path.startswith('file:\\\\\\'): # windows
            path = path[8:] # 8 is len('file:///')
        elif path.startswith('file://'): # nautilus, rox
            path = path[7:] # 7 is len('file://')
        elif path.startswith('file:'): # xffm
            path = path[5:] # 5 is len('file:')
        return path


class ToolbarWidget(gtk.Toolbar):
    '''This represents a toolbar that contains text formatation,
    smilies button, do nudge button, and so on...'''
    
    def __init__(self, parentUI, controller):
        gtk.Toolbar.__init__(self)
        self.parentUI = parentUI
        self.controller = controller
        self.config = controller.config
        self.smilieWindow = SmilieWindow.SmilieWindow(self.controller, \
                                                       self.smilieSelected, \
                                                       None)
        self.FontStyleWindow = FontStyleWindow.FontStyleWindow( \
                                                        self.controller, \
                                                       self.parentUI)

        self.fontFace = gtk.ToolButton()
        self.fontFace.set_stock_id(gtk.STOCK_SELECT_FONT)
        self.fontFace.connect('clicked', self.on_font_face_clicked)
        
        self.fontColor = gtk.ToolButton()
        self.fontColor.set_stock_id(gtk.STOCK_SELECT_COLOR)
        self.fontColor.connect('clicked', self.on_font_color_clicked)

        self.fontStyleButton = gtk.ToolButton()
        self.fontStyleButton.set_label(_('Fontstyle'))
        self.fontStyleButton.set_stock_id(gtk.STOCK_BOLD)
        self.fontStyleButton.connect('clicked', self.showFontStyleWindow)

        self.smilieButton = gtk.ToolButton()
        self.smilieButton.set_label(_('Smilie'))
        self.smilieButton.connect('clicked', self.showSmilieWindow)
        
        self.nudgeButton = gtk.ToolButton()
        self.nudgeButton.set_label(_('Nudge'))
        self.nudgeButton.connect('clicked', self.doNudge)

        self.inviteButton = gtk.ToolButton()
        self.inviteButton.set_label(_('Invite'))
        self.inviteButton.set_stock_id(gtk.STOCK_ADD)
        self.inviteButton.connect('clicked', self.showInviteDialog)
        
        #clear conversation toolbar button
        imgclear = gtk.Image()
        imgclear.set_from_stock(gtk.STOCK_CLEAR, gtk.ICON_SIZE_LARGE_TOOLBAR)
        self.clearButton = gtk.ToolButton(imgclear, _('Clear Conversation'))
        self.clearButton.connect('clicked', self.clearOutputText)
        
        #send a file
        self.sendfileButton = gtk.ToolButton()
        self.sendfileButton.set_label(_('Send File'))
        self.sendfileButton.set_stock_id(gtk.STOCK_GOTO_TOP)
        self.sendfileButton.connect('clicked', self.sendFileClicked)

        # Add buttons to the toolbar
        self.insert(self.fontFace, -1)
        self.insert(self.fontColor, -1)
        self.insert(self.fontStyleButton, -1)
        
        self.insert(gtk.SeparatorToolItem(), -1)
        
        self.insert(self.smilieButton, -1)
        self.insert(self.nudgeButton, -1)

        self.insert(gtk.SeparatorToolItem(), -1)

        self.insert(self.inviteButton, -1)
        self.insert(self.sendfileButton, -1)
        
        self.insert(gtk.SeparatorToolItem(), -1)
        
        self.insert(self.clearButton, -1)

        # Set tooltips to buttons
        self.tooltips = gtk.Tooltips()
        self.tooltips.enable()

        self.fontFace.set_tooltip(self.tooltips, _('Font selection'))
        self.fontColor.set_tooltip(self.tooltips, _('Font color selection'))
        self.fontStyleButton.set_tooltip(self.tooltips, _('Font styles'))
        self.smilieButton.set_tooltip(self.tooltips, _('Insert a smilie'))
        self.nudgeButton.set_tooltip(self.tooltips, _('Send nudge'))
        self.inviteButton.set_tooltip(self.tooltips, \
            _('Invite a friend to the conversation'))
        self.clearButton.set_tooltip(self.tooltips, _('Clear conversation'))
        self.sendfileButton.set_tooltip(self.tooltips, _('Send a file'))
        
        self.show_all()
        
    def setFontBold(self, value):
        self.FontStyleWindow.buttonBold.set_active(value)
    
    def setFontUnderline(self, value):
        self.FontStyleWindow.buttonUnderline.set_active(value)
    
    def setFontStrike(self, value):
        self.FontStyleWindow.buttonStrike.set_active(value)
    
    def setFontItalic(self, value):
        self.FontStyleWindow.buttonItalic.set_active(value)
    
    def on_font_face_clicked(self, *args):
        '''Called when user clicks on the font selection button
        in the toolbar'''
        self.parentUI.parentConversation.parentConversationWindow.changeFont()
    
    def on_font_color_clicked(self, *args):
        ''' Called when user clicks on the color selection button in the toolbar'''
        self.parentUI.parentConversation.parentConversationWindow.changeColor()

    def set_small(self, value):
        '''sets the icons size to small if value is True'''

        if value:
            size = gtk.ICON_SIZE_MENU
        else:
            size = gtk.ICON_SIZE_LARGE_TOOLBAR

        imgSmilie = gtk.Image()
        imgNudge = gtk.Image()
        
        nudge = self.controller.theme.getImage('nudge')
        grin = self.controller.theme.getSmiley(':D')
        if isinstance(grin, gtk.gdk.PixbufAnimation):
            grin = grin.get_static_image()
        
        grin = resizePixbuf(grin, *gtk.icon_size_lookup(size))
        nudge = resizePixbuf(nudge, *gtk.icon_size_lookup(size))
        imgSmilie.set_from_pixbuf(grin)
        imgNudge.set_from_pixbuf(nudge)
        self.smilieButton.set_icon_widget(imgSmilie)
        self.nudgeButton.set_icon_widget(imgNudge)
        
        # set the global icons size 
        settings = self.get_settings()
        settings.set_long_property('gtk-toolbar-icon-size', size, \
            'emesene:ConversationUI')
    
    def smilieSelected(self, smilie=None):
        '''this method is called when the user click a smilie in the
        smiliewindow'''

        if smilie:
            self.parentUI.input.appendInputText(smilie)
        self.parentUI.input.grabFocus()
        
    def showSmilieWindow(self, *args):
        '''this method is called when the user click the smilie button'''
        self.smilieWindow.show()
        
    def showFontStyleWindow(self, *args):
        '''this method calls the font styles window'''
        self.FontStyleWindow.show()
    
    def showInviteDialog(self, *args):
        '''this method is called when the user click the invite button'''
        if not self.controller or not self.controller.msn:
            return

        conversation = self.parentUI.parentConversation
        window = self.parentUI.parentConversation.parentConversationWindow

        window.show_invite_dialog(conversation)

    def clearOutputText(self, *args):
        self.parentUI.parentConversation.textBuffer.set_text('')
        
    def doNudge(self, *args):
        '''this method is called when the user click the nudge button'''
        self.parentUI.parentConversation.doNudge() 

    def sendFileClicked(self, *args):
        if not self.controller or not self.controller.msn:
            return

        win = self.parentUI.parentConversation.parentConversationWindow
        if win is not None:
            win.send_file_dialog()

    def update(self, *args):
        '''this method disables some buttons on switchboard error'''
        conversation = self.parentUI.parentConversation
        if conversation.switchboard.status == 'error' \
         or not self.controller or not self.controller.msn:
            self.nudgeButton.set_sensitive(False)
            self.inviteButton.set_sensitive(False)
            self.sendfileButton.set_sensitive(False)
            self.smilieButton.set_sensitive(False)
        else:
            self.nudgeButton.set_sensitive(True)
            self.inviteButton.set_sensitive(True)
            self.sendfileButton.set_sensitive(True)
            self.smilieButton.set_sensitive(True)

class InviteWindow(gtk.Dialog):
    '''This class represent a list where the user can pick wich users 
    he want to invite to a conversation'''

    def __init__(self, controller, father, onlineUsers, 
                    usersInConversation, theme, callback):
        '''Constructor callback is called when a user is selected with 
        double click'''
        gtk.Dialog.__init__(self, _('Invite'), father,
                     gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                     (gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT, \
                     gtk.STOCK_ADD, gtk.RESPONSE_ACCEPT))

        self.set_default_size(400, 300)
        self.callback = callback
        users = onlineUsers
        
        self.controller = controller
        
        for i in usersInConversation.keys():
            if users.has_key(i):
                del users[i]

        self.userList = UserList.UserList(self.controller, theme,
            self.controller.config, False)
        #self.userList.connect('item-selected', self.userListClicked)

        self.selection = self.userList.get_selection()

        if gtk.gtk_version >= (2, 10, 0):
            self.userList.set_rubber_banding(True)
            self.userList.set_property("rubber-banding", True)
        
            self.selection.set_mode(gtk.SELECTION_MULTIPLE)
        
        group = emesenelib.ContactData.Group(_('Users'))

        for contact in controller.msn.contactManager.contacts.values():
            if contact and contact.status != 'FLN' and \
               contact.email not in usersInConversation:
                group.setUser(contact.email, contact)
            
        self.userList.fill({group.name : group})
        
        self.scroll = gtk.ScrolledWindow()
        self.scroll.set_policy(gtk.POLICY_AUTOMATIC , gtk.POLICY_AUTOMATIC)
        self.scroll.add_with_viewport(self.userList)
        
        self.connect('response', self.invite)
        self.connect('delete-event', self.close)
        
        tipText = _("For multiple select hold CTRL key and click")
        self.tipLabel = gtk.Label()
        self.tipLabel.set_markup("<small><i>" + tipText + "</i></small>")
        
        self.vbox.set_spacing(3)
        self.vbox.pack_start(self.scroll)
        self.vbox.pack_start(self.tipLabel, False, False)
        self.vbox.show_all()
        
    def invite(self, dialog, response_id):
        '''get the user in the userlist and call the callback'''

        try:
            if response_id == gtk.RESPONSE_ACCEPT:
                #self.userListClicked(self.userList, self.userList.getSelected())
                
                model, iter = self.selection.get_selected_rows()
                for i in range(len(iter)):
                    print model[iter[i][0]][2]
                    self.callback(model[iter[i][0]][2])
                self.close()
                return True
            else:
                self.close()
                return False
        except Exception, e:
            print e
            
    def userListClicked(self, userList, t):

        if t and t[0] == 'user':
            self.callback(t[1].email)
            self.close()    

    def close(self, *args):
        '''close the window'''

        self.hide()

class FtBarWidget(gtk.HBox):
    '''bar which represents active file transfers'''
    
    def __init__(self, controller, parentUI, parentConversation):
        
        gtk.HBox.__init__(self)
        
        self.set_spacing(3)
        
        self.controller = controller
        self.parentUI = parentUI
        self.parentConversation = parentConversation
        
        self.hbox = gtk.HBox()
        self.hbox.set_spacing(3)
        
        self.layout = gtk.Layout()
        self.layout.put(self.hbox, 0, 0)
        self.layout.set_size(self.hbox.get_allocation().width, \
                               self.hbox.get_allocation().height + 100)
        
        self.current = 0
        self.speed = 5
        self.page = 0
        self.twidth = 150
        self.num_transfers = 0
        
        arrowLeft = gtk.Arrow(gtk.ARROW_LEFT, gtk.SHADOW_IN)
        arrowRight = gtk.Arrow(gtk.ARROW_RIGHT, gtk.SHADOW_IN)
        self.btnLeft = gtk.Button()
        self.btnLeft.add(arrowLeft)
        self.btnLeft.set_sensitive(False)
        self.btnLeft.set_relief(gtk.RELIEF_NONE)
        self.btnLeft.connect("clicked", self.onbtnLeft)
        
        self.btnRight = gtk.Button()
        self.btnRight.add(arrowRight)
        self.btnRight.set_relief(gtk.RELIEF_NONE)
        self.btnRight.connect("clicked", self.onbtnRight)
        
        self.pack_start(self.btnLeft, False, False)
        self.pack_start(self.layout)
        self.pack_start(self.btnRight, False, False)
        
        
    def add(self, transfer):
        self.newBar = FtWidget(self.controller, self, transfer)
        transfer.ui = self.newBar
        self.hbox.pack_start(self.newBar, False, False)
        self.num_transfers += 1
        self.set_no_show_all(False)
        self.show_all()
    
    def onbtnLeft(self, widget):
        self.twidth = self.newBar.get_allocation().width
        self.page -= 1
        self.dest = -self.twidth * self.page
        gobject.timeout_add(5, self.onSmoothLeft)
    
    def onbtnRight(self, widget):
        self.twidth = self.newBar.get_allocation().width
        self.btnLeft.set_sensitive(True)
        self.page += 1
        self.dest = -self.twidth * self.page
        gobject.timeout_add(5, self.onSmoothRight)
    
    def onSmoothRight(self, *args):
        if self.current > self.dest:
            self.current -= self.speed
            self.layout.move(self.hbox, self.current, 0)
            return True
        return False
    
    def onSmoothLeft(self, *args):
        if self.current < self.dest:
            self.current += self.speed
            self.layout.move(self.hbox, self.current, 0)
            return True
        
        if self.dest == 0: self.btnLeft.set_sensitive(False)
        return False
        

class FtWidget(gtk.HBox):
    '''this class represents the ui widget for one filetransfer'''
    def __init__(self, controller, transferBar, transfer):
        gtk.HBox.__init__(self)
        
        self.controller = controller
        self.transferBar = transferBar
        self.transfer = transfer

        self.eventBox = gtk.EventBox()
        self.progress = gtk.ProgressBar()
        self.progress.set_ellipsize(pango.ELLIPSIZE_END)
        
        self.eventBox.add(self.progress)
        self.pack_start(self.eventBox, False, False)
        
        self.buttons = []
        self.show_all()
        self.tooltip = FileTransferTooltip(self.eventBox, self.transfer)

        self.eventBox.connect('event', self.onProgressBarEvent)
        
        self.updateProgress()
        self.stateChanged()

    def onProgressBarEvent(self, widget, event):
        if event.type == gtk.gdk.BUTTON_PRESS and event.button == 1 and \
                self.transfer.state == self.transfer.RECEIVED:
            self.transfer.open()
    
    def updateProgress(self):
        if self.transfer.state == self.transfer.RECEIVED:
            self.progress.set_fraction(1)  # 100%
        else:
            self.progress.set_fraction(self.transfer.getFraction())
        self.progress.set_text(self.transfer.getFilename())
        self.tooltip.update()

    def stateChanged(self):
        state = self.transfer.state

        # remove existing buttons        
        for button in self.buttons:
            self.remove(button)
        
        self.buttons = []
        
        if state == self.transfer.WAITING and self.transfer.sender != 'Me':
            button = gtk.Button(None, None)
            button.set_image(self.getButtonImage(gtk.STOCK_APPLY))
            button.connect('clicked', self.onAcceptClicked)
            self.buttons.append(button)
        
        if state in (self.transfer.RECEIVED, self.transfer.FAILED):
            button = gtk.Button(None, None)
            button.set_image(self.getButtonImage(gtk.STOCK_CLOSE))
            button.connect('clicked', self.onCloseClicked)
            self.buttons.append(button)
        
        if state == self.transfer.WAITING or state == self.transfer.TRANSFERING:
            cancelButton = gtk.Button(None, None)
            cancelButton.connect('clicked', self.onCancelClicked)
            cancelButton.set_image(self.getButtonImage(gtk.STOCK_CANCEL))
            self.buttons.append(cancelButton)
        
        for button in self.buttons:
            self.pack_start(button, False, False)
        
        self.show_all()
        self.updateProgress()

    def getButtonImage(self, stockImage):
        img = gtk.Image()
        img.set_from_stock(stockImage, gtk.ICON_SIZE_MENU)
        return img

    def onCancelClicked(self, widget):
        self.transfer.cancel()
        self.transferBar.hbox.remove(self)
        self.transferBar.num_transfers -= 1
        if self.transferBar.num_transfers == 0:
            self.transferBar.hide()
        
    def onAcceptClicked(self, widget):
        self.transfer.accept()
        
    def onCloseClicked(self, widget):
        self.transfer.remove()
        self.transferBar.hbox.remove(self)
        self.transferBar.num_transfers -= 1
        if self.transferBar.num_transfers == 0:
            self.transferBar.hide()

DELAY = 500

class FileTransferTooltip(gtk.Window):
    '''Class that implements the filetransfer tooltip'''

    def __init__(self, parentWidget, transfer):

        gtk.Window.__init__(self, gtk.WINDOW_POPUP)

        self.transfer = transfer
        
        self.set_name('gtk-tooltips')
        self.set_position(gtk.WIN_POS_MOUSE)
        self.set_resizable(False)
        self.set_border_width(4)
        self.set_app_paintable(True)
        
        self.image = gtk.Image()
        self.details = gtk.Label()
        self.details.set_alignment(0, 0)
        
        self.table = gtk.Table(3, 2, False)
        self.table.set_col_spacings(5)
        
        self.addLabel(_('Status:'), 0, 1, 0, 1)
        self.addLabel(_('Average speed:'), 0, 1, 1, 2)
        self.addLabel(_('Time elapsed:'), 0, 1, 2, 3)
        
        self.status = gtk.Label()
        self.speed = gtk.Label()
        self.elapsed = gtk.Label()
        
        self.addLabel('', 1, 2, 0, 1, self.status)
        self.addLabel('', 1, 2, 1, 2, self.speed)
        self.addLabel('', 1, 2, 2, 3, self.elapsed)

        vbox = gtk.VBox(False, 5)
        vbox.pack_start(self.details, False, False)
        vbox.pack_start(self.table, False, False)
        
        hbox = gtk.HBox(False, 5)
        hbox.pack_start(self.image, False, False)
        hbox.pack_start(vbox, True, True)
        
        self.add(hbox)
    
        self.connect('expose-event', self.on_expose_event)
        parentWidget.connect('enter-notify-event', self.on_motion)
        parentWidget.connect('leave-notify-event', self.on_leave)

        self.mouseOverWidget = False

    def addLabel(self, s, left, right, top, bottom, label = None):
        if label == None:
            label = gtk.Label(s)
        
        label.set_alignment(0, 0)
        self.table.attach(label, left, right, top, bottom)

    def on_motion(self, view, event):
        self.mouseOverWidget = True
        eventCoords = (event.x_root, event.y_root, int(event.y))
        gobject.timeout_add(DELAY, self.show_tooltip, \
                                            view, eventCoords)

    def show_tooltip(self, view, origCoords):
        # tooltip is shown after a delay, so we have to check 
        # if mouse is still over parent widget
        if not self.mouseOverWidget:
            return
        
        pixbuf = self.transfer.getPreviewImage()
        
        #amsn sends a big. black preview? :S
        if pixbuf != None and pixbuf.get_height() <= 96 and pixbuf.get_width() <= 96:
            self.image.set_from_pixbuf(pixbuf)

        # set the location of the tooltip
        x, y = self.computePosition(origCoords, view.window)
        self.move(x, y)
        self.update()
        self.show_all()
        return False

    def update(self):
        self.details.set_markup('<b>' + self.transfer.getFilename() + '</b>')
        bps = self.transfer.getAverageSpeed()
        seconds = self.transfer.getElapsedTime()
        received, total = self.transfer.getBytes()
        
        percentage = int(self.transfer.getFraction() * 100)
        self.status.set_text('%d%% (%d/%d KB)' % (percentage,  int(received) / 1024, int(total) / 1024))
        self.elapsed.set_text('%.2d:%.2d' % (int(seconds / 60), int(seconds % 60)))
        self.speed.set_text('%.2f KiB/s' % (float(bps) / 1024.0))

    def on_leave(self, view, event):
        self.mouseOverWidget = False
        self.hide()

    # display a border around the tooltip
    def on_expose_event(self, tooltip_window, event):
        width, height = tooltip_window.get_size()
        tooltip_window.style.paint_flat_box(tooltip_window.window, \
                                            gtk.STATE_NORMAL, gtk.SHADOW_OUT, \
                                            None, tooltip_window, 'tooltip', \
                                            0, 0, width, height)
        
    def computePosition(self, origCoords, viewWindow):
        x_root, y_root, origY = origCoords
        currentY = viewWindow.get_pointer()[1]
        
        width, height = self.get_size()
        s_width, s_height = gtk.gdk.screen_width(), gtk.gdk.screen_height()

        x = int(x_root) - width/2
        if currentY >= origY:
            y = int(y_root) + 24
        else:
            y = int(y_root) + 6

        # check if over the screen
        if x + width > s_width:
            x = s_width - width
        elif x < 0:
            x = 0

        if y + height > s_height:
            y = y - height - 24
        elif y < 0:
            y = 0
            
        return (x, y)



# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

'''module to handle p2p sessions on msn protocol'''

import os
import time
import random
import base64
import struct
import gobject

import common
import Msnobj
import msn_slp

common.debugFlag = True

def contains(string, substring):
    '''return True if string contains substring'''
    return (string.find(substring) != -1)
    
# we use the max from sys.maxint in a 32 bit machine because
# in a 64 bit machine it is bigger and can give some troubles
def random_number(minimum = 0, maximum = 2147483647):
    '''return a random number between minimum and maximum'''
    return random.randint(minimum, maximum)
 
class Base(gobject.GObject):
    '''a base class for P2P handlers
    this class has all the methods to create the different
    messages and to check if a message belong to a handler'''

    __gsignals__ = { 
        # message, session_id
        'msnp2p-message-ready':
        (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, 
            (gobject.TYPE_PYOBJECT, ) * 2),
        
        # flags, file, footer, bye (bool)
        'msnp2p-file-ready':
        (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_PYOBJECT, ) * 4),

        # message
        'debug':
        (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_PYOBJECT, ))
    }

    # workarounds for pylint lack of support of builtin subclassing
    # just to get cleaner output
    emit = gobject.GObject.emit
    connect = gobject.GObject.connect
    disconnect = gobject.GObject.disconnect
    
    # seconds alive until disconnecting automatically
    MAX_TIME_ALIVE = 60*3

    FOOTER = '\0\0\0\0'
    DP_FOOTER = '\0\0\0\1'
    FILE_FOOTER = '\0\0\0\2'
    INK_FOOTER = '\0\0\0\3'
    CE_FOOTER = '\0\0\0\x0c'

    # these must be replaced on subclassing
    temporary_session = None
    signals_p2p = [] # ['msnp2p-message-received']
    signals_self = [] # ['msnp2p-message-ready']
    
    def __init__(self, p2p):
        '''from_ is the sender and to the receiver email'''
        gobject.GObject.__init__(self)

        # the moment when its created
        self.time_stamp = time.time()
        
        # list of identifiers of connected signals
        self.signal_ids = []

        self.from_ = p2p.manager.msn.user
        self.to_ = p2p.mail
        
        # this values are just the most common ones
        self.via = msn_slp.random_id()
        self.cseq = '0'
        self.call_id = msn_slp.random_id()
        self.max_forwards = '0'
        self.content_type = ''
        self.content_length = ''
        self.euf_guid = ''
        self.session_id = str(random_number(50000))
        self.app_id = ''
        self.context = ''

        self.data = None

        p2p.emit('new-p2p-session', self)

    def check_time_limit(self):
        '''check if the time alive is greater than MAX_TIME_ALIVE
        if it is, return True to let the gc destroy the object'''

        if self.temporary_session and \
            time.time() > self.time_stamp + Base.MAX_TIME_ALIVE:

            self.debug('max time alive exceeded', 'p2p')
            return True

    def debug(self, message, _channel=''):
        '''Emits the debug signal and displays the message in the terminal'''
        self.emit('debug', message)
        common.debug(message, 'p2p')
        
    def is_data(self, bin):
        '''return True if the header is from a data message of
        this p2p session'''

        if bin.flag != bin.ACK_FLAG and \
           bin.session_id == int(self.session_id):
            return True

        return False

    def is_last_data(self, bin):
        '''return True if it is the last data message'''

        if self.is_data(bin) and \
            bin.data_offset + bin.message_length == bin.total_data_size:
            return True

    def connect_handler(self, p2p):
        '''connects signals, yay'''

        # the following may be confusing, but who cares? just don't touch
        # i use strings here to avoid saving a reference
        targets = ('p2p', self.signals_p2p), ('self', self.signals_self)

        for owner, signals in targets:
            if owner == 'p2p':
                src, dest = p2p, self
            else:
                src, dest = self, p2p

            for signal in signals:
                funcname = 'on_' + signal.replace("-", "_")
                identifier = src.connect(signal, getattr(dest, funcname))
                self.signal_ids.append((owner, identifier))

    def disconnect_handler(self, p2p):
        '''disconnects itself allowing garbage collection'''
        self.debug('disconnecting p2p handler', 'p2p')

        for owner, identifier in self.signal_ids:
            if owner == 'p2p':
                src = p2p
            else:
                src = self
            src.disconnect(identifier)


class Sender(Base):
    '''A class that sends p2p content based on msnobjs (with dpm)
    This is unbeliabely short, heh.'''

    temporary_session = True
    
    signals_p2p = []
    signals_self = ['msnp2p-message-ready', 'msnp2p-file-ready']

    def __init__(self, p2p, bin_header, slp, data):
        Base.__init__(self, p2p)

        self.data = data
        self.via = slp.via
        self.call_id = slp.call_id
        self.session_id = slp.body['SessionID']
        
        self.connect_handler(p2p)
        self.emit('msnp2p-message-ready', msn_slp.ok(self), 0)
        self.emit('msnp2p-message-ready', '\0\0\0\0', self.session_id) #dpm
        self.emit('msnp2p-file-ready', bin_header.EACH_FLAG, self.data,
            Base.DP_FOOTER, self.on_transfer_complete)

    def on_transfer_complete(self, p2p):
        self.emit('msnp2p-message-ready', msn_slp.bye(self), 0)
        self.disconnect_handler(p2p)


class Receiver(Base):
    '''a class that request data to other client and receive it'''

    temporary_session = True

    signals_p2p = ['msnp2p-message-received']
    signals_self = ['msnp2p-message-ready']

    def __init__(self, p2p, msnobj):
        '''p2p is a P2PUser, msnobj is a Msnobj'''

        if p2p.last_requested_msnobj == msnobj:
            return
        
        Base.__init__(self, p2p)
        
        p2p.last_requested_msnobj = msnobj
        self.msnobj = msnobj

        if self.msnobj.type == Msnobj.Msnobj.DISPLAY_PICTURE:
            self.app_id = '1' 
        elif self.msnobj.type == Msnobj.Msnobj.CUSTOM_EMOTICON:
            self.app_id = '12' 

        self.connect_handler(p2p)

        context = base64.b64encode(str(msnobj) + '\0')
        self.emit('msnp2p-message-ready', msn_slp.invite(self, context), 0)
    
    def on_msnp2p_message_received(self, p2p, bin, slp, message):
        '''method called when P2PManager receives a msnp2p message
        we check if it is from this session, if it is, we process it
        if we are finished or something went wrong, we disconnect the signal
        '''
        if self.check_time_limit():
            self.disconnect_handler(p2p)
        else:
            self.debug('receiver checking', 'p2p')
            if int(self.session_id) == int(bin.session_id):
                body = message[48:-4]

                if body == '\0\0\0\0':   # ignore dpm
                    return
                
                if len(body) != bin.total_data_size:
                    self.debug('corrupt data, not all pieces received')
                elif self.msnobj.type == Msnobj.Msnobj.CUSTOM_EMOTICON:
                    self.debug('custom-emoticon-data-received', 'p2p')
                    p2p.emit('custom-emoticon-data-received', 
                        self.msnobj, body, self.to_)
                elif self.msnobj.type == Msnobj.Msnobj.DISPLAY_PICTURE:
                    self.debug('display-picture-received', 'p2p')
                    p2p.emit('display-picture-received', 
                        self.msnobj, body, self.to_)
                else:
                    self.debug('no CE or DP? %s' % self.msnobj.type, 'p2p')

                self.disconnect_handler(p2p)


class DCHandler(Base):
    '''A class that handles direct connect invites
    After the automatic ack massacre, most code here was removed,
    so handling ACK MSNSLP messages'''

    temporary_session = False

    signals_p2p = ['msnp2p-message-received']
    signals_self = ['msnp2p-message-ready']

    def __init__(self, p2p, bin_header, slp):
        '''this processes the first invite, subsequent invites should '''
        Base.__init__(self, p2p)

        self.via = slp.via
        self.call_id = slp.call_id
        if 'SessionID' in slp.body:
            self.session_id = slp.body['SessionID']
        else:
            self.session_id = 0
        
        self.connect_handler(p2p)
        self.tempp2p = p2p
        
        self.handle_invite(bin_header, slp)
            
    def handle_invite(self, bin_header, slp):
        '''Handles a invite message'''

        if slp.content_type == msn_slp.SLPMessage.CONTENT_TRANS_REQ:
            # params after \r\n\r\n:
            #Bridges: TRUDPv1 TCPv1 SBBridge TURNv1
            #NetID: -1689699384
            #Conn-Type: Port-Restrict-NAT
            #TCP-Conn-Type: Port-Restrict-NAT
            #UPnPNat: false
            #ICF: false
            #Hashed-Nonce: {44C4DB18-98AC-2025-1404-1998BEE1C437}
            #SessionID: 10958830
            #SChannelState: 0
            #Capabilities-Flags: 1

            message = msn_slp.ok(self)
            message.content_type = msn_slp.SLPMessage.CONTENT_TRANS_RESP
            message.body = {
                'Bridge': 'TCPv1',
                'Listening': 'false',
                'Nonce': '{00000000-0000-0000-0000-000000000000}',
            }
            self.emit('msnp2p-message-ready', message, 0)

        elif slp.content_type == msn_slp.SLPMessage.CONTENT_TRANS_RESP:
            # the second invite is a TRANSRESP. body:
            # 
            # Bridge: TCPv1
            # Listening: true
            # Conn-Type: Direct-Connect
            # TCP-Conn-Type: Direct-Connect
            # Nonce: {A10AE850-D3DD-4A4A-A69D-C41F01D41F94}
            # IPv4Internal-Addrs: 5.127.##.## 169.254.##.### 201.255.###.##
            # IPv4Internal-Port: 3660
            #  <alt>
            # IPv4Internal-Addrs: 190.183.##.##
            # IPv4Internal-Port: 1434
            # SessionID: 278241011
            # SChannelState: 0
            # Capabilities-Flags: 1

            # this is auto acked
            pass

    def on_msnp2p_message_received(self, p2p, bin, slp, message):
        self.debug('DC Handler checking')
        if 'SessionID' in slp.body:
            session_id = slp.body['SessionID']
        else:
            session_id = bin.session_id

        if session_id == self.session_id or slp.call_id == self.call_id:
            if slp.method.startswith('ACK'):

                addrs = []
                for field in ('External', 'Internal'):
                    fieldname = 'IPv4' + field + 'AddrsAndPorts'
                    if fieldname in slp.body:
                        addrs.extend(slp.body[fieldname].strip().split())
                
                self.debug("Got ACK, i'll connect to " + repr(addrs))
                
                #p2p.get_switchboard()
                #p2p.switchboard.emit('message', 'p2p', '',
                #    'Received address list ' + repr(addrs), '', '')
                #time.sleep(10)
                
            elif slp.method.startswith('BYE'):
                self.disconnect_handler(p2p)


class FTSender(Base):
    '''a class that request data to other client and receive it'''
    
    temporary_session = False
    
    signals_p2p = ['msnp2p-message-received', 'file-transfer-canceled']
    signals_self = ['msnp2p-message-ready', 'msnp2p-file-ready']

    def __init__(self, p2p, filename):
        Base.__init__(self, p2p)
        
        self.app_id = '2' 
        
        self.context = FTContext()
        self.context.filename = filename
        #self.context.preview = base64.b64encode( self.data )
        self.context.file_size = os.stat(filename).st_size
        self.data = open(filename, "rb")

        self.connect_handler(p2p)
        self.current_transfer = 0

        message = msn_slp.invite(self, base64.b64encode(str(self.context)),
            msn_slp.SLPMessage.EUFGUID_FILE)

        self.emit('msnp2p-message-ready', message, 0)
        
    def on_msnp2p_message_received(self, p2p, bin, slp, message):
        '''method called when P2PManager receives a msnp2p message
        we check if it is from this session, if it is, we process it
        if we are finished or something went wrong, we disconnect the signal
        '''
        
        if 'SessionID' in slp.body:
            session_id = int(slp.body['SessionID'])
        else:
            session_id = int(bin.session_id)

        self.debug("FTSender checking")

        if session_id == int(self.session_id):
            self.debug("Mine!")
            if slp.method == msn_slp.SLPMessage.OK_STATUS:
                self.debug("Accepted")
                common.debug('sending data', 'p2p')
                p2p.emit('file-transfer-accepted', int(self.session_id),
                    self.context, 'Me')
                self.emit('msnp2p-file-ready', bin.FILE_FLAG | \
                    bin.EACH_FLAG | bin.STORAGE_FLAG, self.data,
                    Base.FILE_FOOTER, self.on_transfer_complete)
                
            elif slp.method == msn_slp.SLPMessage.DECLINE_STATUS or \
                 slp.method.startswith('BYE'):
                
                self.debug("Cancelled")
                # notify the user
                p2p.emit('transfer-failed', int(self.session_id), 'cancelled')
                self.disconnect_handler(p2p)
            else:
                self.debug("wtf is " + str(slp.method))
        elif slp.call_id == self.call_id and slp.method.startswith("BYE"):
            self.debug("file transfer canceled", 'p2p')
            p2p.emit('transfer-failed', int(self.session_id), 'cancelled')
            self.disconnect_handler(p2p)
            
    def on_transfer_complete(self, p2p):
        p2p.emit('file-transfer-complete', int(self.session_id),
            self.context, None, 'Me')
        self.emit('msnp2p-message-ready', msn_slp.bye(self), 0)
        self.disconnect_handler(p2p)

    def on_file_transfer_canceled(self, p2p, session, context, sender):
        '''callback for P2PUser file-transfer-canceled, sends a decline
        message canceling the transfer'''
        if int(session) == int(self.session_id):
            self.debug('canceling FT')
            self.emit('msnp2p-message-ready', msn_slp.bye(self), 0)
            self.disconnect_handler(p2p)
    
    def disconnect_handler(self, p2p):
        Base.disconnect_handler(self, p2p)
        if self.current_transfer and \
           self.current_transfer in p2p.outgoing_pending_messages:
            # this stops the send data callback
            del p2p.outgoing_pending_messages[self.current_transfer]
            self.current_transfer = 0

class FTReceiver(Base):
    '''a class to accept and handle an incoming file transfer'''

    temporary_session = False

    signals_p2p = ['msnp2p-message-received', 'msnp2p-file-received',
        'file-transfer-accepted', 'file-transfer-canceled']
    signals_self = ['msnp2p-message-ready']

    def __init__(self, p2p, context, slp, bin_header):
        '''context, slp and bin_header are from the INVITE message'''
        Base.__init__(self, p2p)

        self.context = context

        self.via = slp.via
        self.call_id = slp.call_id
        self.session_id = int(slp.body['SessionID'])
        
        self.connect_handler(p2p)

        p2p.start_new_conv()
        p2p.emit('file-transfer-invite', int(self.session_id), \
            self.context, self.from_)
        
    def on_file_transfer_accepted(self, p2p, session, context, sender):
        '''callback for P2PUser file-transfer-accepted, sends a 200ok
        message starting the transfer'''
        if int(session) == int(self.session_id) and \
           context == self.context and \
           sender == self.from_:

            # accept the file
            self.debug('accepting FT')
            self.emit('msnp2p-message-ready', msn_slp.ok(self), 0)

    def on_file_transfer_canceled(self, p2p, session, context, sender):
        '''callback for P2PUser file-transfer-canceled, sends a decline
        message canceling the transfer'''
        if int(session) == int(self.session_id):
            self.debug('canceling FT')
            self.emit('msnp2p-message-ready', msn_slp.bye(self), 0)
            self.disconnect_handler(p2p)

    def on_msnp2p_message_received(self, p2p, bin, slp, message):
        '''method called when P2PManager receives a msnp2p message
        we check if it is from this session, if it is, we process it
        if we are finished or something went wrong, we disconnect the signal
        '''
        
        self.debug('FT Receiver checking', 'p2p')
        if 'SessionID' in slp.body:
            session_id = int(slp.body['SessionID'])
        else:
            session_id = int(bin.session_id)

        if session_id == int(self.session_id):
            if slp.method == msn_slp.SLPMessage.DECLINE_STATUS or \
               slp.method.startswith('BYE'):
                
                self.debug("file transfer canceled", 'p2p')
                p2p.emit('transfer-failed', int(self.session_id), 'cancelled')
                self.disconnect_handler(p2p)

            if bin.flag == bin.CANCEL_FLAG:
                self.debug('cancelled on TLP')
                p2p.emit('transfer-failed', int(self.session_id), 'error')
                self.disconnect_handler(p2p)

        elif slp.call_id == self.call_id and slp.method.startswith("BYE"):
            self.debug("file transfer canceled", 'p2p')
            p2p.emit('transfer-failed', int(self.session_id), 'cancelled')
            self.disconnect_handler(p2p)
            
    def on_msnp2p_file_received(self, p2p, bin, rc):
        '''Method called when the file is received succesfully'''
        self.debug('FTReceiver checking file')
        if self.is_data(bin) and bin.flag & bin.FILE_FLAG:
            self.debug('file received')
            
            # file received successfully
            self.debug('file-transfer-complete')
            p2p.emit('file-transfer-complete', int(self.session_id),
                self.context, rc, self.from_)
            self.disconnect_handler(p2p)



class FTContext(object):
    '''this class represents a File Transfer Context'''

    #FORMAT = '<LLQL520s30sL64s'  # perl?
    FORMAT = '<LLQL520s30sL'
    PORTABLE_FORMAT = '<LQL520s'  # note: without header
    HEADER_FORMAT = '<L'

    UNKNOWN_FT = 0xffffffff
    UNKNOWN_BG = 0xfffffffe
    RUBBISH = base64.b64encode("rubbish")

    TYPE_NO_PREVIEW = 0
    TYPE_PREVIEW = 1
    TYPE_BACKGROUND = 4
    
    def __init__(self, data=None):
        '''Constructor'''
        
        self.header_length = struct.calcsize(FTContext.FORMAT)
                                     # ..dw
        self.msnc = 2                # dw (3 == msn7)
        self.file_size = 0           # qw
        self.__data_type = 0         # dw
        self.__filename = ''         # 520 byte string, utf-16-le
        self.rubbish = ''            # 30 byte string, optional base64
        self.unknown1 = 0            # dw (see UNKNOWN_(BG|FT))

        # set properties
        self.data_type = FTContext.TYPE_NO_PREVIEW
        self.filename = ''

        self.preview = ''

        if data != None:
            self.fill(data)
            
    def __str__(self):
        '''return the representation of this object'''

        return struct.pack(FTContext.FORMAT,
            self.header_length,
            self.msnc,
            self.file_size,
            self.data_type,
            self.__filename,
            self.rubbish,
            self.unknown1) + self.preview
    
    def set_data_type(self, dword):
        '''property setter for data_type
        when it's set as background, the rubbish and unknown1 fields
        are set to the corresponding values'''

        self.__data_type = dword
        if dword == FTContext.TYPE_BACKGROUND:
            self.rubbish = FTContext.RUBBISH
            self.unknown1 = FTContext.UNKNOWN_BG
        else:
            self.rubbish = ''
            self.unknown1 = FTContext.UNKNOWN_FT

    def get_data_type(self):
        '''property getter for data_type'''
        return self.__data_type

    data_type = property(fset=set_data_type, fget=get_data_type)

    def set_filename(self, data):
        '''set the value of filename'''
        data = os.path.basename(data)
        self.__filename = data.encode('utf-16-le', 'replace').ljust(520, '\0')
        
    def get_filename(self):
        '''get the value of filename'''
        return self.__filename.decode('utf-16-le', 'replace').replace('\0', '')

    filename = property(fset=set_filename, fget=get_filename)
    
    def print_fields(self):
        '''print the binary fields'''
        print
        print 'Header length:   ' + str(self.header_length)
        print 'msnc:            ' + str(self.msnc)
        print 'File size:       ' + str(self.file_size)
        print 'Type:            ' + str(self.data_type)
        print 'Filename:        ' + str(self.filename)
        print
        
    def fill(self, data):
        '''fill the object with the data provided'''
        self.header_length = struct.unpack(FTContext.HEADER_FORMAT, data[:4])[0]
        portable_size = struct.calcsize(FTContext.PORTABLE_FORMAT)
        header = data[4:portable_size + 4]
        
        # from __future__ import braces
        (   
            self.msnc,
            self.file_size,
            self.data_type,
            self.__filename,
        ) = struct.unpack(FTContext.PORTABLE_FORMAT, header)

        self.preview = data[self.header_length:]



# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
'''a group of methods and callable classes to validate values, used on 
ConfigDialog to validate the inputs, but also can be used on other places'''

import os

def not_empty(value):
    '''return True if not empty'''
    if value:
        return True
    else:
        return False

def is_file(value):
    '''return True if is file'''
    return os.path.isfile(value)

def is_dir(value):
    '''return True if is directory'''
    return os.path.isdir(value)

def is_int(value):
    '''return True if is an int or can be converted to an int'''
    try:
        int(value)
        return True
    except ValueError:
        return False

def is_float(value):
    '''return True if is a float or can be converted to a float'''
    try:
        float(value)
        return True
    except ValueError:
        return False

def is_bool(value):
    '''return True if the value is or can be converted to a bool'''
    try:
        bool(value)
        return True
    except ValueError:
        return False

def path_exists(value):
    '''return True if path exists'''
    return os.path.exists(value)

def readable(value):
    '''return True if the file exists and is readable'''
    if is_file(value) and os.access(value, os.R_OK):
        return True

    return False

def writable(value):
    '''return True if the file exists and is writeable'''
    if is_file(value) and os.access(value, os.W_OK):
        return True

    return False

class Range(object):
    '''a callable class that return True if the value parameter is 
    whitin the range including the extremes'''

    def __init__(self, min_val, max_val):
        '''class constructor'''
        self.min_val = min_val
        self.max_val = max_val

    def __call__(self, value):
        '''check if value is whitin the range, includint the extremes'''
        if self.min_val <= value <= self.max_val:
            return True

        return False

class InItems(object):
    '''a callable class that return True if the value is in the list of items
    '''

    def __init__(self, items):
        '''class constructor'''
        self.items = items

    def __call__(self, value):
        '''check if value is in items'''
        return value in self.items




# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

'''a module that defines the api of objects that display dialogs'''

import os
import gtk
import gobject

import Avatar

import abstract.stock as stock

_avatar_chooser = None

def window_add_image(window, stock_id):
    '''add a stock image as the first element of the window.hbox'''

    image = gtk.image_new_from_stock(stock_id, gtk.ICON_SIZE_DIALOG)
    window.hbox.pack_start(image, False)
    image.show()

    return image

def window_add_button(window, stock_id, label=''):
    '''add a button to the window'''

    button = gtk.Button(label, stock=stock_id)
    window.bbox.pack_start(button, True, True)
    button.show()

    return button

def window_add_label(window, text):
    '''add a label with the text (as pango) on the window'''

    label = gtk.Label()
    label.set_use_markup(True)
    label.set_markup('<span>' + \
        text + "</span>")
    window.hbox.pack_start(label, True, True)
    label.show()

    return label

def close_cb(widget, event, window, response_cb, *args):
    '''default close callback, call response_cb with args if it's not
    None'''

    if response_cb:
        response_cb(*args)

    window.hide()

def default_cb(widget, window, response_cb, *args):
    '''default callbacks, call response_cb with args if it's not
    None'''

    if response_cb:
        response_cb(*args)

    window.hide()

def entry_cb(widget, window, response_cb, *args):
    '''callback called when the entry is activated, it call the response
    callback with the stock.ACCEPT and append the value of the entry
    to args'''
    args = list(args)
    args.append(window.entry.get_text())

    if response_cb:
        if type(widget) == gtk.Entry:
            response_cb(stock.ACCEPT, *args)
        else:
            response_cb(*args)

    window.hide()

def add_contact_cb(widget, window, response_cb, response):
    '''callback called when a button is selected on the add_contact dialog'''
    contact = window.entry.get_text()
    group = window.combo.get_model().get_value(window.combo.get_active_iter(), 0)

    window.hide()
    response_cb(response, contact, group)

def set_picture_cb(widget, window, response_cb, response, path):
    '''callback called when a button is selected on the set_picture dialog'''
    window.hide()
    response_cb(response, path)

def common_window(message, stock_id, response_cb, title):
    '''create a window that displays a message with a stock image'''
    window = new_window(title, response_cb)
    window_add_image(window, stock_id)
    window_add_label(window, message)

    return window

def message_window(message, stock_id, response_cb, title):
    '''create a window that displays a message with a stock image
    and a close button'''
    window = common_window(message, stock_id, response_cb, title)
    add_button(window, gtk.STOCK_CLOSE, stock.CLOSE, response_cb,
        default_cb)

    return window

def entry_window(message, text, response_cb, title, *args):
    '''create a window that contains a label and a entry with text set
    and selected, and two buttons, accept, cancel'''
    window = new_window(title, response_cb)
    window_add_label(window, message)

    entry = gtk.Entry()
    entry.set_text(text)
    entry.select_region(0, -1)

    entry.connect('activate', entry_cb, window, response_cb, *args)

    window.hbox.pack_start(entry, True, True)
    add_button(window, gtk.STOCK_CANCEL, stock.CANCEL, response_cb, 
        entry_cb, *args)
    add_button(window, gtk.STOCK_OK, stock.ACCEPT, response_cb, 
        entry_cb, *args)

    setattr(window, 'entry', entry)

    entry.show()

    return window

def add_button(window, gtk_stock, stock_id, response_cb,
    callback, *args):
    '''add a button and connect the signal'''
    button = gtk.Button(stock=gtk_stock)
    window.bbox.pack_start(button, True, True)
    button.connect('clicked', callback, window, response_cb,
        stock_id, *args)

    button.show()

    return button
    
def new_window(title, response_cb, *args):
    '''build a window with the default values and connect the common
    signals, return the window'''

    window = gtk.Window()
    window.set_title(title)
    window.set_default_size(150, 100)
    window.set_position(gtk.WIN_POS_CENTER)
    window.set_border_width(8)

    vbox = gtk.VBox(spacing=4)
    hbox = gtk.HBox(spacing=4)
    bbox = gtk.HButtonBox()
    bbox.set_spacing(4)
    bbox.set_layout(gtk.BUTTONBOX_END)

    vbox.pack_start(hbox, True, True)
    vbox.pack_start(bbox, False)

    window.add(vbox)

    setattr(window, 'vbox', vbox)
    setattr(window, 'hbox', hbox)
    setattr(window, 'bbox', bbox)

    args = list(args)
    args.insert(0, stock.CLOSE)
    window.connect('delete-event', close_cb, window,
        response_cb, *args)

    vbox.show_all()

    return window

def error(message, response_cb=None, title=_("Error!")):
    '''show an error dialog displaying the message, this dialog should
    have only the option to close and the response callback is optional
    since in few cases one want to know when the error dialog was closed,
    but it can happen, so return stock.CLOSE to the callback if its set'''
    message_window(message, gtk.STOCK_DIALOG_ERROR, response_cb, 
        title).show()

def warning(message, response_cb=None, title=_("Warning")):
    '''show a warning dialog displaying the messge, this dialog should
    have only the option to accept, like the error dialog, the response
    callback is optional, but you have to check if it's not None and
    send the response (that can be stock.ACCEPT or stock.CLOSE, if
    the user closed the window with the x)'''
    message_window(message, gtk.STOCK_DIALOG_WARNING, response_cb, 
        title).show()

def information(message, response_cb=None, 
                        title=_("Information"),):
    '''show a warning dialog displaying the messge, this dialog should
    have only the option to accept, like the error dialog, the response
    callback is optional, but you have to check if it's not None and
    send the response (that can be stock.ACCEPT or stock.CLOSE, if
    the user closed the window with the x)'''
    message_window(message, gtk.STOCK_DIALOG_INFO, response_cb, 
        title).show()

def exception(message, response_cb=None, title=_("Exception"),):
    '''show the message of an exception on a dialog, useful to
    connect with sys.excepthook'''
    window = new_window(title, response_cb)
    label = window_add_label(window, message)
    label.set_selectable(True)
    add_button(window, gtk.STOCK_CLOSE, stock.CLOSE, response_cb,
        default_cb)

    window.show()

def yes_no(message, response_cb, *args):
    '''show a confirm dialog displaying a question and two buttons:
    Yes and No, return the response as stock.YES or stock.NO or
    stock.CLOSE if the user closes the window'''
    window = common_window(message, gtk.STOCK_DIALOG_QUESTION, 
        response_cb, _("Confirm"))
    add_button(window, gtk.STOCK_YES, stock.YES, response_cb, 
        default_cb, *args)
    add_button(window, gtk.STOCK_NO, stock.NO, response_cb, 
        default_cb, *args)

    window.show()

def yes_no_cancel(message, response_cb, *args):
    '''show a confirm dialog displaying a question and three buttons:
    Yes and No and Cancel, return the response as stock.YES, stock.NO,
    stock.CANCEL or stock.CLOSE if the user closes the window'''
    window = common_window(message, gtk.STOCK_DIALOG_QUESTION, 
        response_cb, _("Confirm"))
    add_button(window, gtk.STOCK_YES, stock.YES, response_cb, 
        default_cb, *args)
    add_button(window, gtk.STOCK_NO, stock.NO, response_cb, 
        default_cb, *args)
    add_button(window, gtk.STOCK_CANCEL, stock.CANCEL, response_cb, 
        default_cb, *args)

    window.show()

def accept_cancel(message, response_cb, *args):
    '''show a confirm dialog displaying information and two buttons:
    Accept and Cancel, return stock.ACCEPT, stock.CANCEL or 
    stock.CLOSE'''
    window = common_window(message, gtk.STOCK_DIALOG_QUESTION, 
        response_cb, _("Confirm"))
    add_button(window, gtk.STOCK_OK, stock.ACCEPT, response_cb, 
        default_cb, *args)
    add_button(window, gtk.STOCK_CANCEL, stock.CANCEL, response_cb, 
        default_cb, *args)

    window.show()

def contact_added_you(accounts, response_cb, 
                            title=_("User invitation")):
    '''show a dialog displaying information about users
    that added you to their userlists, the accounts parameter is
    a tuple of mails that represent all the users that added you,
    the way you confirm (one or more dialogs) doesn't matter, but
    you should call the response callback only once with a tuple
    like: ((mail1, stock.YES), (mail2, stock.NO), (mail3, stock.CANCEL))
    YES means add him to your userlist, NO means block him, CANCEL
    means remind me later.'''
    raise NotImplementedError("This method isn't implemented")

def add_contact(groups, group, response_cb, title=_("Add user")):
    '''show a dialog asking for an user address, and (optional)
    the group(s) where the user should be added, the response callback
    receives the response type (stock.ADD, stock.CANCEL or stock.CLOSE)
    the account and a tuple of group names where the user should be 
    added (give a empty tuple if you don't implement this feature, 
    the controls are made by the callback, you just ask for the email, 
    don't make any control, you are just implementing a GUI! :P'''
    window = new_window(title, response_cb)
    label = gtk.Label(_("Account"))
    label_align = gtk.Alignment(0.0, 0.5)
    label_align.add(label)
    entry = gtk.Entry()
    group_label = gtk.Label(_("Group"))
    group_label_align = gtk.Alignment(0.0, 0.5)
    group_label_align.add(group_label)
    combo = gtk.combo_box_new_text()

    combo.append_text("")

    groups = list(groups)
    groups.sort()

    selected = 0

    for (index, group_name) in enumerate(groups):
        combo.append_text(group_name)

        if group == group_name:
            selected = index + 1

    combo.set_active(selected)

    table = gtk.Table(2, 2)
    table.attach(label_align, 0, 1, 0, 1)
    table.attach(entry, 1, 2, 0, 1)
    table.attach(group_label_align, 0, 1, 1, 2)
    table.attach(combo, 1, 2, 1, 2)
    table.set_row_spacings(2)
    table.set_col_spacings(8)

    window.hbox.pack_start(table, True, True)

    add_button(window, gtk.STOCK_CANCEL, stock.CANCEL, response_cb, 
        add_contact_cb)
    add_button(window, gtk.STOCK_OK, stock.ACCEPT, response_cb, 
        add_contact_cb)

    setattr(window, 'entry', entry)
    setattr(window, 'combo', combo)

    entry.connect('activate', add_contact_cb, window, response_cb,
        stock.ACCEPT)
    window.show_all()

def add_group(response_cb, title=_("Add group")):
    '''show a dialog asking for a group name, the response callback
    receives the response (stock.ADD, stock.CANCEL, stock.CLOSE)
    and the name of the group, the control for a valid group is made
    on the controller, so if the group is empty you just call the
    callback, to make a unified behaviour, and also, to only implement
    GUI logic on your code and not client logic
    cb args: response, group_name'''
    window = entry_window(_("Group name"), '', response_cb, title)
    window.show()

def set_nick(nick, response_cb, title=_("Change nick")):
    '''show a dialog asking for a new nick and displaying the current
    one, the response_cb receives the old nick, the new nick, 
    and the response (stock.ACCEPT, stock.CANCEL or stock.CLOSE)
    cb args: response, old_nick, new_nick'''
    window = entry_window(_("New nick"), nick, response_cb, title,
    nick)
    window.show()

def set_message(message, response_cb, 
    title=_("Change personal message")):
    '''show a dialog asking for a new personal message and displaying 
    the current one, the response_cb receives the old personal message
    , the new personal message and the response 
    (stock.ACCEPT, stock.CANCEL or stock.CLOSE)
    cb args: response, old_pm, new_pm'''
    window = entry_window(_("New personal message"), 
        message, response_cb, title, message)
    window.show()

def set_picture(path_current, cache_path,  
        response_cb, title=_("Change picture")):
    '''show a dialog asking for the display picture and return the selected
    one, path_current is the path of the current selected picture.
    cache_path is a folder where the avatars are loaded and saved
     Return the response and a new path or None if no new picture 
    is selected'''
    global _avatar_chooser

    def _on_hide(window):
        global _avatar_chooser
        _avatar_chooser = None

    if not _avatar_chooser:
        _avatar_chooser = AvatarChooser(response_cb, 
            path_current, cache_path)

        _avatar_chooser.connect('hide', _on_hide)
        _avatar_chooser.show()
    else:
        _avatar_chooser.grab_focus()

def set_custom_emoticon(ce_dir, response_cb):
    '''show a dalog to create a custome emotion, ce_dir is the
    directory where the image chooser dialog will open'''
    CEChooser(ce_dir, response_cb).show()

def rename_group(name, response_cb, title=_("Rename group")):
    '''show a dialog with the group name and ask to rename it, the
    response callback receives stock.ACCEPT, stock.CANCEL or stock.CLOSE
    the old and the new name.
    cb args: response, old_name, new_name
    '''
    window = entry_window(_("New group name"), name, response_cb, 
        title, name)
    window.show()

def set_contact_alias(account, alias, response_cb, 
                        title=_("Set alias")):
    '''show a dialog showing the current alias and asking for the new
    one, the response callback receives,  the response 
    (stock.ACCEPT, stock.CANCEL, stock.CLEAR <- to remove the alias 
    or stock.CLOSE), the account, the old and the new alias.
    cb args: response, account, old_alias, new_alias'''
    alias = alias or ''
    window = entry_window(_("Contact alias"), alias, response_cb, 
        title, account, alias)
    add_button(window, gtk.STOCK_CLEAR, stock.CLEAR, response_cb,
        entry_cb, account, alias)
    window.show()

def about_dialog(name, version, copyright, comments, license, website,
    authors, translators, logo_path):
    '''show an about dialog of the application:
    * title: the title of the window
    * name: the name of the appliaction
    * version: version as string
    * copyright: the name of the copyright holder
    * comments: a description of the application
    * license: the license text
    * website: the website url
    * authors: a list or tuple of strings containing the contributors
    * translators: a string containing the translators
    '''

    def close_about(widget, response_id):
        if response_id == gtk.RESPONSE_CANCEL:
            widget.destroy()

    about = gtk.AboutDialog()
    about.set_name(name)
    about.set_version(version)
    about.set_copyright(copyright)
    about.set_comments(comments)
    about.connect('response', close_about)
    about.set_license(license)
    about.set_website(website)

    about.set_authors(authors)
    about.set_translator_credits(translators)
    icon = gtk.image_new_from_file(logo_path)
    about.set_icon(icon)
    about.set_logo(icon)
    about.run()

def _callback(*args):
    '''a callback that print all the arguments'''
    print(args)

def _test():
    '''a test method'''
    '''error("Error!", _callback)
    warning("Warning!", _callback)
    information("Information", _callback)
    yes_no("to_be || !to_be", _callback, "foo")
    yes_no_cancel("to_be or not to_be", _callback, "bar")
    accept_cancel("Silly question", _callback, "baz")
    add_contact(('c', 'b', 'a'), _callback)
    add_group(_callback)
    set_nick("my other nick is a ferrari", _callback)
    set_message("too personal", _callback)
    rename_group("Groupal", _callback)
    set_contact_alias("dude@live.com", "some dude", _callback)
    exception("*implodes*")'''
    set_picture('', _callback)
    gtk.main()

#TODO: rewrite me on the contact_added_you method
import Widgets
class AddBuddy(gtk.Window):
    '''Confirm dialog informing that someone has added you
    ask if you want to add him to your contact list'''
    
    def __init__(self, controller):
        '''Constructor. Packs widgets'''
        gtk.Window.__init__(self)
        
        self.mails = []  # [(mail, nick), ...]
        self.pointer = 0
        self.controller = controller

        # window
        self.set_title(_("Add contact"))
        self.set_border_width(4)
        self.move(30, 30) # top-left
        self.connect('delete-event', self.cb_delete)
        
        ## widgets

        # main vbox
        self.vbox = gtk.VBox()
        
        # hbox with image, pages, and main text
        self.hbox = gtk.HBox()
        self.hbox.set_spacing(4)
        self.hbox.set_border_width(4)
        
        # the contents of the hbox (image+vboxtext)
        self.image = gtk.Image()
        self.image.set_from_stock(gtk.STOCK_DIALOG_QUESTION, \
            gtk.ICON_SIZE_DIALOG)
        self.imagebox = gtk.HBox()
        self.imagebox.set_border_width(4)
        self.image.set_alignment(0.0, 0.5)
        
        # the vboxtext (pages+text)
        self.vboxtext = gtk.VBox()
        self.pages = self._buildpages()
        self.text = gtk.Label()
        self.text.set_selectable(True)
        self.text.set_ellipsize(3) #pango.ELLIPSIZE_END
        self.text.set_alignment(0.0, 0.0) # top left
        self.text.set_width_chars(60)
        
        # hboxbuttons + button box
        self.hboxbuttons = gtk.HBox()
        self.hboxbuttons.set_spacing(4)
        self.hboxbuttons.set_border_width(4)
        self.buttonbox = gtk.HButtonBox()
        self.buttonbox.set_layout(gtk.BUTTONBOX_END)

        # the contents of the buttonbox
        self.later = gtk.Button()
        self.later.add(gtk.Label(_('Remind me later')))
        self.later.connect('clicked', self.cb_cancel)
        self.profile = gtk.Button()
        self.profile.add(gtk.Label(_('View profile')))
        self.profile.connect('clicked', self.cb_profile)
        self.addbutton = gtk.Button(stock=gtk.STOCK_ADD)
        self.addbutton.connect('clicked', self.cb_add)
        
        ## packing
        self.add(self.vbox)
        self.vbox.pack_start(self.hbox, True, True)
        self.vbox.pack_start(self.hboxbuttons, False, False)

        self.imagebox.pack_start(self.image)
        self.hbox.pack_start(self.imagebox, False, False)
        self.hbox.pack_start(self.vboxtext, True, True)
        self.vboxtext.pack_start(self.pages, False, False)
        self.vboxtext.pack_start(self.text, True, True)

        self.hboxbuttons.pack_start(self.later, False, False)
        self.hboxbuttons.pack_start(self.profile, False, False)
        self.hboxbuttons.pack_start(self.buttonbox)
        self.buttonbox.pack_start(self.addbutton)

    def _buildpages(self):
        '''Builds hboxpages, that is a bit complex to include in __init__'''
        hboxpages = gtk.HBox()
        
        arrowleft = Widgets.TinyArrow(gtk.ARROW_LEFT)
        self.buttonleft = gtk.Button()
        self.buttonleft.set_relief(gtk.RELIEF_NONE)
        self.buttonleft.add(arrowleft)
        self.buttonleft.connect('clicked', self.switchmail, -1)

        arrowright = Widgets.TinyArrow(gtk.ARROW_RIGHT)
        self.buttonright = gtk.Button()
        self.buttonright.set_relief(gtk.RELIEF_NONE)
        self.buttonright.add(arrowright)
        self.buttonright.connect('clicked', self.switchmail, 1)
        
        self.currentpage = gtk.Label()

        hboxpages.pack_start(gtk.Label(), True, True) # align to right
        hboxpages.pack_start(self.buttonleft, False, False)
        hboxpages.pack_start(self.currentpage, False, False)
        hboxpages.pack_start(self.buttonright, False, False)

        return hboxpages

    def append(self, nick, mail):
        '''Adds a new pending user'''
        self.mails.append((mail, gobject.markup_escape_text(nick)))
        self.update()
        self.show_all()
        self.present()

    def update(self):
        '''Update the GUI, including labels, arrow buttons, etc'''
        try:
            mail, nick = self.mails[self.pointer]
        except IndexError:
            self.hide()
            return

        if nick != mail:
            mailstring = "<b>%s</b>\n<b>(%s)</b>" % (nick, mail)
        else:
            mailstring = '<b>%s</b>' % mail

        self.text.set_markup(mailstring + _(' has added you.\n'
            'Do you want to add him/her to your contact list?'))
        
        self.buttonleft.set_sensitive(True)
        self.buttonright.set_sensitive(True)
        if self.pointer == 0:
            self.buttonleft.set_sensitive(False)
        if self.pointer == len(self.mails) - 1:
            self.buttonright.set_sensitive(False)
            
        self.currentpage.set_markup('<b>(%s/%s)</b>' % \
            (self.pointer + 1, len(self.mails)))
    
    def switchmail(self, button, order):
        '''Moves the mail pointer +1 or -1'''
        if (self.pointer + order) >= 0:
            if (self.pointer + order) < len(self.mails):
                self.pointer += order
            else:
                self.pointer = 0
        else:
            self.pointer = len(self.mails) - 1

        self.update()

    def hide(self):
        '''Called to hide the window'''
        gtk.Window.hide(self)
        self.controller.addBuddy = None

    def cb_delete(self, *args):
        '''Callback when the window is destroyed'''
        self.controller.addBuddy = None
        self.destroy()

    def cb_cancel(self, button):
        '''Callback when the cancel button is clicked'''
        self.mails.pop(self.pointer)
        self.switchmail(None, -1)
        self.update()

    def cb_profile(self, button):
        '''Callback when the view profile button is clicked'''
        self.controller.seeProfile(self.mails[self.pointer][0])

    def cb_add(self, button):
        '''Callback when the add button is clicked'''
        mail, nick = self.mails[self.pointer]
        self.controller.contacts.add(mail)
        self.cb_cancel(None)

class AvatarChooser(gtk.Window):
    '''A dialog to choose an avatar'''

    def __init__(self, response_cb, picture_path='', 
            cache_path='.'):
        '''Constructor, response_cb receive the response number, the new file 
        selected and a list of the paths on the icon view.
        picture_path is the path of the current display picture, 
        '''
        gtk.Window.__init__(self)

        self.response_cb = response_cb
        self.cache_path = cache_path

        self.set_title(_("Avatar chooser"))
        self.set_default_size(600, 400)
        self.set_border_width(4)
        self.set_position(gtk.WIN_POS_CENTER)
        
        self.model = gtk.ListStore(gtk.gdk.Pixbuf, str)
        
        self.view = gtk.IconView(self.model)
        self.view.enable_model_drag_dest([('text/uri-list', 0, 0)], gtk.gdk.ACTION_DEFAULT | gtk.gdk.ACTION_COPY)
        self.view.connect("drag-data-received", self._drag_data_received)
        
        self.view.set_pixbuf_column(0)
        self.view.connect("item-activated", self._on_icon_activated)
    
        scroll = gtk.ScrolledWindow()
        scroll.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
        scroll.set_shadow_type(gtk.SHADOW_IN)
        scroll.add(self.view)
        
        vbox = gtk.VBox(spacing=4)
        side_vbox = gtk.VBox(spacing=4)
        hbox = gtk.HBox(spacing=4)

        hbbox = gtk.HButtonBox()
        hbbox.set_spacing(4)
        hbbox.set_layout(gtk.BUTTONBOX_END)
        
        vbbox = gtk.VButtonBox()
        vbbox.set_spacing(4)
        vbbox.set_layout(gtk.BUTTONBOX_START)

        b_clear = gtk.Button(_("No picture"))
        b_add = gtk.Button(stock=gtk.STOCK_ADD)
        b_remove = gtk.Button(stock=gtk.STOCK_REMOVE)
        b_remove_all = gtk.Button(_("Remove all"))
        b_accept = gtk.Button(stock=gtk.STOCK_OK)
        b_cancel = gtk.Button(stock=gtk.STOCK_CANCEL)

        b_clear.connect('clicked', self._on_clear)
        b_add.connect('clicked', self._on_add)
        b_remove.connect('clicked', self._on_remove)
        b_remove_all.connect('clicked', self._on_remove_all)
        b_accept.connect('clicked', self._on_accept)
        b_cancel.connect('clicked', self._on_cancel)
        self.connect('delete-event', self._on_close)
        
        self.img_current = gtk.Image()
        self.img_current.set_size_request(96, 96)
        frame_current = gtk.Frame(_("Current"))
        frame_current.add(self.img_current)

        hbbox.pack_start(b_clear, False)
        hbbox.pack_start(b_cancel, False)
        hbbox.pack_start(b_accept, False)

        vbbox.pack_start(b_add, False)
        vbbox.pack_start(b_remove, False)
        vbbox.pack_start(b_remove_all, False)

        side_vbox.pack_start(frame_current, False)
        side_vbox.pack_start(vbbox)

        hbox.pack_start(scroll, True, True)
        hbox.pack_start(side_vbox, False, False)

        vbox.pack_start(hbox, True, True)
        vbox.pack_start(hbbox, False)

        vbox.show_all()
        self.add(vbox)

        self.fill()
        self.set_current_picture(picture_path)

    def set_current_picture(self, path):
        '''set the current picture on the frame'''
        if os.path.exists(path):
            pixbuf = gtk.gdk.pixbuf_new_from_file(path)
            self.img_current.set_from_pixbuf(pixbuf)

    def get_selected(self):
        '''return a tuple (pixbuf, path) of the selection, or None'''
        iter = self.get_selected_iter()

        if iter:
            return self.model[iter]

        return None

    def get_selected_iter(self):
        '''return the selected iter or None'''
        if len(self.view.get_selected_items()) > 0:
            item = self.view.get_selected_items()[0]
            return self.model.get_iter(item)

        return None

    def get_iter_from_filename(self, path):
        '''return the iter of a filename or None'''
        for row in self.model:
            (pixbuf, filename) = row
            if os.path.samefile(filename, path):
                return row.iter

        return None
            
    def fill(self):
        '''fill the IconView with avatars from the list of pictures'''

        for path in os.listdir(self.cache_path):
            if not os.path.splitext(path)[0].endswith('_thumb'):
                self.add_picture(os.path.join(self.cache_path, path))
                    
    def is_in_view(self, filename):
        '''return True if filename already on the iconview'''

        if os.name == 'nt':
            # nt doesn't include os.path.samefile
            return False

        for (pixbuf, path) in self.model:
            if os.path.samefile(filename, path):
                return True

        return False
                
            
    def add_picture(self, path):
        '''Adds an avatar into the IconView'''
        try:
            if os.path.exists(path) and os.access(path, os.R_OK)\
                    and not self.is_in_view(path):
                pixbuf = gtk.gdk.pixbuf_new_from_file(path)

                self.model.append([pixbuf, path])
            else:
                print path, 'not readable'
        except gobject.GError:
            print 'image at %s could not be loaded'
            print gobject.GError

    def remove(self, path):
        '''remove the avatar in path'''
        del self.model[self.get_iter_from_filename(path)]
        try:
            os.remove(path)
            parts = os.path.splitext(path)
            os.remove(parts[0] + "_thumb" + parts[1])
        except Exception, e:
            print "could not remove", path
            
    def remove_selected(self):
        '''Removes avatar from a TreeIter'''
        selected = self.get_selected()

        if selected:
            (pixbuf, path) = selected
            self.remove(path)

    def remove_all(self):
        '''remove all the items on the view'''
        for (pixbuf, path) in self.model:
            self.remove(path)
    
        self.model.clear()
    
    def _drag_data_received(self, treeview, context, x, y, selection, info, 
                             timestamp):
        '''method called on an image dragged to the view'''
        urls = selection.data.split('\n')
        for url in urls:
            path = url.replace('file://', '')
            path = path.replace('\r', '')

            # this seems to be an error on ntpath but we take care :S
            try:
                if os.path.exists(path):
                    self.add_picture(path)
            except TypeError, e:
                error(_("Could not add picture:\n %s") % (str(e),))

    def _on_icon_activated(self, *args):
        '''method called when a picture is double clicked'''
        self._on_accept(None)

    def _on_add(self, button):
        '''called when the user select the add button'''
        def _on_image_selected(response, path):
            '''method called when an image is selected'''
            if response == stock.ACCEPT:
                try:
                    av = Avatar.Avatar(path, self.cache_path, resizeDialog=True)
                    filename = av.getImagePath()
                    self.add_picture(av.getImagePath())
                except Exception, e:
                    error(str(e))

        ImageChooser(os.path.expanduser('~'), _on_image_selected).show() 
        
    def _on_remove(self, event):
        '''Removes the selected avatar'''
        self.remove_selected()

    def _on_remove_all(self, button):
        '''Removes all avatars from the cache'''
        def on_response_cb(response):
            '''response callback for the confirm dialog'''
            if response == stock.YES:
                self.remove_all()

        yes_no(_("Are you sure you want to remove all items?"),
            on_response_cb)

    def _on_accept(self, button):
        '''method called when the user clicks the button'''
        selected = self.get_selected()
        filename = ''

        if selected:
            filename = selected[1]

            self.hide()
            self.response_cb(stock.ACCEPT, filename)
        else:
            error(_("No picture selected"))
    
    def _on_cancel(self, button):
        '''method called when the user clicks the button'''
        self.hide()
        self.response_cb(stock.CANCEL, '')
    
    def _on_clear(self, button):
        '''method called when the user clicks the button'''
        self.hide()
        self.response_cb(stock.CLEAR, '')

    def _on_close(self, window, event):
        '''called when the user click on close'''
        self.hide()
        self.response_cb(stock.CLOSE, '')

class ImageChooser(gtk.Window):
    '''a class to select images'''

    def __init__(self, path, response_cb):
        '''class constructor, path is the directory where the
        dialog opens'''
        gtk.Window.__init__(self)

        self.response_cb = response_cb
        
        self.set_title(_("Image Chooser"))
        self.set_default_size(600, 400)
        self.set_border_width(4)
        self.set_position(gtk.WIN_POS_CENTER)

        self.vbox = gtk.VBox(spacing=4)

        self.file_chooser = gtk.FileChooserWidget()
        self.file_chooser.set_current_folder(path)
        
        hbbox = gtk.HButtonBox()
        hbbox.set_spacing(4)
        hbbox.set_layout(gtk.BUTTONBOX_END)
        
        b_accept = gtk.Button(stock=gtk.STOCK_OK)
        b_cancel = gtk.Button(stock=gtk.STOCK_CANCEL)

        b_accept.connect('clicked', self._on_accept)
        b_cancel.connect('clicked', self._on_cancel)
        self.connect('delete-event', self._on_close)
        
        hbbox.pack_start(b_cancel, False)
        hbbox.pack_start(b_accept, False)
        
        vbox = gtk.VBox()
        self.vbox.pack_start(self.file_chooser, True, True)
        
        vbox.add(self.vbox)
        vbox.pack_start(hbbox, False)
        self.add(vbox)
        vbox.show_all()

        self._add_filters()
        self._add_preview()
    
    def _add_filters(self):
        '''
        Adds all the possible file filters to the dialog. The filters correspond
        to the gdk available image formats
        '''
        
        # All files filter
        all_files = gtk.FileFilter()
        all_files.set_name(_('All files'))
        all_files.add_pattern('*')
        
        # All images filter
        all_images = gtk.FileFilter()
        all_images.set_name(_( 'All images'))
        
        filters = []
        formats = gtk.gdk.pixbuf_get_formats()
        for format in formats:
            filter = gtk.FileFilter()
            name = "%s (*.%s)" % (format['description'], format['name'])
            filter.set_name(name)
            
            for mtype in format['mime_types']:
                filter.add_mime_type(mtype)
                all_images.add_mime_type(mtype)
                
            for pattern in format['extensions']:
                tmp = '*.' + pattern
                filter.add_pattern(tmp)
                all_images.add_pattern(tmp)
                
            filters.append(filter)
         
        
        self.file_chooser.add_filter(all_files)
        self.file_chooser.add_filter(all_images)
        self.file_chooser.set_filter(all_images)

        for filter in filters:
            self.file_chooser.add_filter(filter)
    
    def _add_preview(self):
        '''
        Adds a preview widget to the file chooser
        '''
        
        self.image = gtk.Image()
        self.image.set_size_request(128, 128)
        self.image.show()

        self.file_chooser.set_preview_widget(self.image)
        self.file_chooser.set_preview_widget_active(True)

        self.file_chooser.connect('update-preview', self._on_update_preview)
    
    def _on_accept(self, button):
        '''method called when the user clicks the button'''
        filename = get_filename(self)
        if os.path.isfile(filename):
            self.hide()
            self.response_cb(stock.ACCEPT, filename)
        else:
            error(_("No picture selected"))
    
    def _on_cancel(self, button):
        '''method called when the user clicks the button'''
        self.hide()
        self.response_cb(stock.CANCEL, get_filename(self))
    
    def _on_close(self, window, event):
        '''called when the user click on close'''
        self.hide()
        self.response_cb(stock.CLOSE, get_filename(self))
    
    def _on_update_preview(self, filechooser):
        '''
        Updates the preview image
        '''
        hasPreview = False
        
        path = get_preview_filename(self)

        if path:
            # if the file is smaller than 1MB we
            # load it, otherwise we dont
            if os.path.isfile(path) and os.path.getsize(path) <= 1000000:
                try:
                    pixbuf = gtk.gdk.pixbuf_new_from_file(get_filename(self))
                    if pixbuf.get_width() > 128 and pixbuf.get_height() > 128:
                        pixbuf = pixbuf.scale_simple(128, 128, gtk.gdk.INTERP_BILINEAR)
                    self.image.set_from_pixbuf(pixbuf)
                    
                except gobject.GError:
                    self.image.set_from_stock(gtk.STOCK_DIALOG_ERROR, 
                        gtk.ICON_SIZE_DIALOG)
            else:
                self.image.set_from_stock(gtk.STOCK_DIALOG_ERROR, 
                    gtk.ICON_SIZE_DIALOG)

class CEChooser(ImageChooser):
    '''a dialog to create a custom emoticon'''
    SMALL = _("small (16x16)")
    BIG = _("big (50x50)")

    def __init__(self, path, response_cb):
        '''class constructor'''
        ImageChooser.__init__(self, path, None)

        self.response_cb = response_cb

        label = gtk.Label(_("Shortcut"))
        self.shortcut = gtk.Entry()
        self.combo = gtk.combo_box_new_text()

        self.combo.append_text(CEChooser.SMALL)
        self.combo.append_text(CEChooser.BIG)
        self.combo.set_active(0)

        hbox = gtk.HBox()

        hbox.add(label)
        hbox.add(self.shortcut)
        hbox.add(self.combo)

        self.vbox.pack_start(hbox, False)
        hbox.show_all()
        
    def _on_accept(self, button):
        '''method called when the user clicks the button'''
        filename = get_filename(self)
        shortcut = self.shortcut.get_text()
        size = self.combo.get_model().get_value(self.combo.get_active_iter(), 0)

        if os.path.isfile(filename):
            if not shortcut:
                error(_("Empty shortcut"))
            else:
                self.hide()
                self.response_cb(stock.ACCEPT, filename, shortcut, size)
        else:
            error(_("No picture selected"))
    
    def _on_cancel(self, button):
        '''method called when the user clicks the button'''
        self.hide()
        self.response_cb(stock.CANCEL, None, None, None)
    
    def _on_close(self, window, event):
        '''called when the user click on close'''
        self.hide()
        self.response_cb(stock.CLOSE, None, None, None)
            
def get_filename(self):
    '''Shortcut to get a properly-encoded filename from a file chooser'''
    filename = self.file_chooser.get_filename()
    if filename and gtk.gtk_version >= (2, 10, 0):
        return gobject.filename_display_name(filename)
    else:
        return filename

def get_preview_filename(self):
    '''Shortcut to get a properly-encoded preview filename'''
    filename = self.file_chooser.get_preview_filename()
    if filename and gtk.gtk_version >= (2, 10, 0):
        return gobject.filename_display_name(filename)
    else:
        return filename

if __name__ == '__main__':
    #_test()
    ac = AvatarChooser(_callback, '/home/mariano/default/icon96.png', '/home/mariano/default/', [])
    ac.show()
    #ic = ImageChooser('/home/mariano/default', _callback)
    #ic.show()
    #ce = CEChooser('/home/mariano/default', _callback)
    #ce.show()

    gtk.main()



# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

# Audacious subplugin for CurrentSong plugin
# by rescbr, Patrick Dessalle

VERSION = '0.2'
IFACE_NAME = 'org.atheme.audacious'
IFACE_PATH = '/org/atheme/audacious'
# Documentation for the dbus: http://svn.atheme.org/audacious/trunk/src/audacious/objects.xml

import os
import commands

import CurrentSong
 
class Audacious( CurrentSong.DbusBase ):

    def __init__( self ):
        # init for this class
        CurrentSong.DbusBase.__init__( self, IFACE_NAME, self.setInterface )
        
        try:
            self.iface
        except:
            self.iface = None # We don't run a recent Audacious with dbus support
            self.playingNow = '' # We'll use that variable to hold the filename as returned by Audacious
        
    def setInterface( self ):
        self.iface = self.bus.get_object( IFACE_NAME, IFACE_PATH )
        
    def isPlaying( self ):
        if self.iface:
            return self.iface.Status() == "playing" #Note: self.iface.Playing() returns True even if it's paused
        else:
            return commands.getoutput("audtool playback-status") == "playing"
        
    def getStatus( self ):
        '''check if everything is OK to start the plugin
        return a tuple whith a boolean and a message
        if OK -> ( True , 'some message' )
        else -> ( False , 'error message' )'''
        
        if os.name != 'posix':
            return ( False, _( 'This plugin only works in posix systems' ) ) #no posix here
   
        if not self.is_on_path( 'audtool' ) and not self.iface:
            return ( False, _( 'audtool is not on the path and you don\'t have dbus enabled.' ) ) #no audtool here
        
        return ( True, 'Ok' ) #ok, run baby!

    def getCurrentSong( self ):
        if not self.iface:
            if self.isPlaying():
                song = commands.getoutput( "audtool current-song" )
                fsong = '\\0Music\\01\\0' + song + '\\0\\0'
                return fsong
            else:
                return ''
        else: #Use the parseStyle from the super class
            return self.parseStyle()
        
    def check( self ):
        if self.iface:
            if self.isPlaying():
                songPosition = self.iface.Position()
                artist = self.iface.SongTuple(songPosition, "artist") 
	        title = self.iface.SongTuple(songPosition, "title") 
	        album = self.iface.SongTuple(songPosition, "album") 
	        if self.artist != artist or self.title != title or self.album != album:
	            self.artist = artist
	            self.title = title
	            self.album = album
	            self.filename = self.iface.SongTitle(songPosition)
	            return True
	    else: # Just stopped, paused, ... set the data as empty strings
	        if self.artist != "" or self.title != "" or self.album != "":
	            self.artist = ""
	            self.title = ""
	            self.album = ""
	            self.filename = ""
	            return True
        
        
        elif self.isPlaying():
            currentSong = self.getCurrentSong()
            if self.playingNow != currentSong:
                self.playingNow = currentSong
                return True
        elif self.playingNow != "": # Just stopped, paused ... set to an empty string
            self.playingNow = ""
            return True
        
        # Not playing or no changes
        return False



# -*- coding: utf-8 -*-

#   This file is a plugin for emesene.
#
#    Dbus Emesene Plugin is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Dbus Emesene Plugin is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import gtk
import Plugin
from gobject import timeout_add, source_remove

class PlusColorPanel(gtk.Window):  
    
    def __init__(self):
        gtk.Window.__init__(self)
        self.set_decorated(False)
        self.set_type_hint(gtk.gdk.WINDOW_TYPE_HINT_DIALOG)        
        self.set_default_size(300, 300)
        self.set_border_width(5)
        self.set_position(gtk.WIN_POS_MOUSE)
        
        self.connect('delete-event' , self.on_delete_event)
        self.connect('leave-notify-event' , self.on_leave_notify_event)
        self.connect('enter-notify-event' , self.on_enter_notify_event)
        
        self.vbox = gtk.VBox()
       
        self.colorCodes = [
'ffffff', '000000', '00007D', '009100', 'FF0000', '7D0000', '9A009A', 'FC7D00', 
'FFFF00', '00FC00', '009191', '00FFFF', '1E1EFC', 'FF00FF', '7D7D7D', 'D2D2D2', 
'E7E6E4', 'cfcdd0', 'ffdea4', 'ffaeb9', 'ffa8ff', 'c1b4fc', 'bafbe5', 'ccffa3', 
'fafda2', 'b6b4b7', 'a2a0a1', 'f9c152', 'ff6d66', 'ff62ff', '6c6cff', '68ffc3', 
'000000', 'f9ff57', '858482', '6e6d7b', 'ffa01e', 'F92411', 'FF1EFF', '1E29FF', 
'7dffa5', '60f913', 'fff813', '5e6464', '4b494c', 'd98812', 'eb0505', 'de00de', 
'0000d3', '03cc88', '59d80d', 'd4c804', '333335', '18171c', '944e00', '9b0008', 
'980299', '01038c', '01885f', '389600', '9a9e15', '473400', '4d0000', '5f0162', 
'000047', '06502f', '1c5300', '544d05']

        self.toolbar = gtk.Toolbar()
        self.toolbar.set_orientation(gtk.ORIENTATION_HORIZONTAL)
        self.toolbar.set_style(gtk.TOOLBAR_ICONS)
        self.toolbar.connect('enter-notify-event' , self.on_enter_notify_event)
        
        self.tooltips = gtk.Tooltips()
        
        self.btnBold = gtk.ToolButton(gtk.STOCK_BOLD)
        self.btnBold.connect("clicked", self.select_format, 'b')
        self.btnBold.set_tooltip(self.tooltips, _("Bold"))
        self.toolbar.add(self.btnBold)
        
        self.btnItalic = gtk.ToolButton(gtk.STOCK_ITALIC)
        self.btnItalic.connect("clicked", self.select_format, 'i')
        self.btnItalic.set_tooltip(self.tooltips, _("Italic"))
        self.toolbar.add(self.btnItalic)
        
        self.btnUnderline = gtk.ToolButton(gtk.STOCK_UNDERLINE)
        self.btnUnderline.connect("clicked", self.select_format, 'u')
        self.btnUnderline.set_tooltip(self.tooltips, _("Underline"))
        self.toolbar.add(self.btnUnderline)
        
        self.btnStrike = gtk.ToolButton(gtk.STOCK_STRIKETHROUGH)
        self.btnStrike.connect("clicked", self.select_format, 's')
        self.btnStrike.set_tooltip(self.tooltips, _('Strike'))
        self.toolbar.add(self.btnStrike)

        self.toolbar.add(gtk.SeparatorToolItem())
        
        self.btnSelectColor = gtk.ToggleToolButton(gtk.STOCK_COLOR_PICKER)
        self.btnSelectColor.connect('clicked', self.on_color_select_clicked)
        self.btnSelectColor.set_tooltip(self.tooltips, _("Select custom color"))
        self.toolbar.add(self.btnSelectColor)
        
        self.toolbar.add(gtk.SeparatorToolItem())
        
        self.btnBackground = gtk.ToggleToolButton(gtk.STOCK_SELECT_COLOR)
        self.btnBackground.set_tooltip(self.tooltips, _("Enable background color"))
        self.toolbar.add(self.btnBackground)
        self.tooltips.enable()
        self.vbox.pack_start(self.toolbar, False)
        
        self.table = gtk.Table(8)
        self.table.set_row_spacings(3)
        self.table.set_col_spacings(3)

        x = 0
        y = 0
        for i in self.colorCodes:
            if x == 8:
                x = 0
                y += 1
            try:
                self.btnColor = gtk.Button()
                color = self.btnColor.get_colormap().alloc_color("#"+ i)
                style = self.btnColor.get_style().copy()
                style.bg[gtk.STATE_NORMAL] = color
                self.btnColor.set_style(style)
                self.btnColor.connect('clicked', self.select_color, self.colorCodes.index(i))
                self.btnColor.connect('enter-notify-event' , self.on_enter_notify_event)
                self.table.attach(self.btnColor, x, x+1, y, y+1)
            except:
                x -= 1
            x += 1
            
        self.vbox.pack_start(self.table)
        
        self.buffer = gtk.TextBuffer()
        self.closed = False
        self.add(self.vbox)
        self.vbox.show_all()
        self.tag = None
        
    def on_leave_notify_event(self, *args):
        if not self.tag and not self.closed:
            self.tag = timeout_add(500, self.hide)
            
    def on_enter_notify_event(self, *args):
        if self.tag:
            source_remove(self.tag)
            self.tag = None
            
    def on_delete_event(self , *args):
        self.hide()
        self.closed = True
        return True
    
    def hide(self):
        gtk.Window.hide(self)
        if (self.tag):
            source_remove(self.tag)
            self.tag = None
        self.closed = True
        
    def show(self):
        gtk.Window.show(self)
        self.closed = False
        
    def select_color(self, widget=None, color=None):
        if self.btnBackground.get_active(): tag = 'a'
        else: tag = 'c'
        
        if self.buffer.get_has_selection():
            start, end = self.buffer.get_selection_bounds()
            selectedtext = self.buffer.get_text(start, end)
            self.buffer.delete(start, end)
            self.buffer.insert(start, "[%s=%s]%s[/%s]" %(tag, color, selectedtext, tag))
        else:
            self.buffer.insert_at_cursor("[%s=%s][/%s]" %(tag, color, tag))
            cursor=self.buffer.get_iter_at_mark(self.buffer.get_insert())
            cursor.backward_chars(4)
            self.buffer.place_cursor(cursor)
        self.btnBackground.set_active(False)
        self.hide()
    
    def select_format(self, widget, tag):
        if self.buffer.get_has_selection():
            start, end = self.buffer.get_selection_bounds()
            selectedtext = self.buffer.get_text(start, end)
            self.buffer.delete(start, end)
            self.buffer.insert(start, "[%s]%s[/%s]" %(tag, selectedtext, tag))
        else:
            self.buffer.insert_at_cursor("[%s][/%s]" %(tag, tag))
            cursor=self.buffer.get_iter_at_mark(self.buffer.get_insert())
            cursor.backward_chars(4)
            self.buffer.place_cursor(cursor)
        self.hide()

    def on_color_select_clicked(self, *args):
        colorDialog = gtk.ColorSelectionDialog('Choose a color')
        colorDialog.colorsel.set_has_palette(True)
        response = colorDialog.run()
        if response == gtk.RESPONSE_OK:
            color = colorDialog.colorsel.get_current_color()
            red = color.red >> 8
            green = color.green >> 8
            blue = color.blue >> 8
            color = '#%02X%02X%02X' % (red , green , blue)
            self.select_color(color=color)
        colorDialog.destroy()


class PlusButton(gtk.ToolButton):
    def __init__(self, conversation):
        
        gtk.ToolButton.__init__(self, gtk.STOCK_COLOR_PICKER)
        self.pluswindow = PlusColorPanel()
        self.pluswindow.buffer = conversation.ui.input.inputBuffer
    
    def show_plus_window(self, widget=None, data=None):
        self.pluswindow.show()
    
    def clean(self):
        self.pluswindow.destroy()
        self.destroy()

class MainClass(Plugin.Plugin):
    
    def __init__(self, controller, msn):
        Plugin.Plugin.__init__(self, controller, msn)
        self.description = _('A panel for applying Messenger Plus formats to the text') 
        self.authors = { 'mg' : 'themgzzy at gmail dot com' }
        self.displayName = _('Plus Color Panel')
        self.name = 'PlusColorPanel'
        
    def start(self):
        '''start the plugin'''

        for conversation in self.getOpenConversations():
            self.add_button(conversation=conversation)
            
        self.signalopen = self.controller.conversationManager.connect('new-conversation-ui', self.add_button)
        self.signalclose = self.controller.conversationManager.connect('close-conversation-ui', self.remove_button)
        self.enabled = True
    
    def stop(self):
        
        for conversation in self.getOpenConversations():
            self.remove_button(conversation=conversation)
                    
        self.controller.conversationManager.disconnect(self.signalopen)
        self.controller.conversationManager.disconnect(self.signalclose)
        self.enabled = False
        
    def check(self):
        '''
        check if everything is OK to start the plugin
        return a tuple whith a boolean and a message
        if OK -> ( True , 'some message' )
        else -> ( False , 'error message' )
        '''
        
        return (True, 'Ok')

    def add_button(self, conversationmanager=None, conversation=None, win=None):
        button = PlusButton(conversation)
        button.connect('clicked', button.show_plus_window)
        button.show()
        toolbar = conversation.ui.input.toolbar
        toolbar.add(button)
    
    def remove_button(self, conversationmanager=None, conversation=None, win=None):
        for button in conversation.ui.input.toolbar.get_children():
            if type(button) == PlusButton:
                conversation.ui.input.toolbar.remove(button)
                button.pluswindow.destroy()
                button.destroy()



# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


import os
import gobject
import gettext

ERROR = ''

try:
    import win32api
    import win32gui
except:
    ERROR = 'cannot import needed modules'


import CurrentSong
from emesenecommon import *

class Winamp( CurrentSong.CurrentSong ):

    def __init__( self ):
        CurrentSong.CurrentSong.__init__( self )
        self.winamp = None
        self.isRunning()
        self.currentSong = ''
        
    def isPlaying( self ):
        if self.winamp and win32api.SendMessage(self.winamp, 0x400, 0, 104) != 1:
            return False
         
        return True
        
    def isRunning( self ):
        try:
            self.winamp = win32gui.FindWindow('Winamp v1.x', None)
            return True
        except: 
            return False

    def getCurrentSong( self ):
        return self.currentSong

    def check( self ):
        if not self.isRunning(): return False
        
        if self.isPlaying():
            newCurrentSong = 'Winamp\\0Music\\01\\0{0}\\0%s\\0\\0' % \
                win32gui.GetWindowText( self.winamp ).replace( ' - Winamp' , '' )
            print newCurrentSong
        else:
            newCurrentSong = ''

        if self.currentSong != newCurrentSong:
            self.currentSong = newCurrentSong
            return True
            
        return False
    
    def getStatus( self ):
        '''
        check if everything is OK to start the plugin
        return a tuple whith a boolean and a message
        if OK -> ( True , 'some message' )
        else -> ( False , 'error message' )
        '''
        
        if os.name != 'nt':
            return ( False, 'This plugin only works on windows systems' )
        
        if ERROR != '':
            return ( False, ERROR )
        
        return ( True, 'Ok' )



# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import gtk

from Plugin import Plugin

from emesenecommon import *

class MainClass(Plugin):
    
    def __init__(self, controller, msn):
        '''
        Contructor
        '''
        
        Plugin.__init__(self, controller, msn, 100)
        self.alpha = 2
        self.description = _('Shakes the window when a nudge is received.')
        self.authors = {unicode('Yguaratã C. Cavalcanti','latin-1').
            encode('latin-1') : 'yguarata at gmail dot com' }
        self.website = 'http://www.yguarata.org'
        self.displayName = _('Window Trembling Nudge')
        self.name = 'WindowTremblingNudge'
        self.shakeWindowId = None
        self.controller = controller
    
    def start(self):
        self.enabled = True
        self.shakeWindowId = self.connect('nudge-received', 
            self.shakeWindow)
        
    def stop(self):
        '''stop the plugin'''
        self.disconnect(self.shakeWindowId)
        self.enabled = False
        
    def check(self):
        '''
        check if everything is OK to start the plugin
        return a tuple whith a boolean and a message
        if OK -> (True , 'some message')
        else -> (False , 'error message')
        '''
        
        return (True, 'Ok')
        
    def shakeWindow(self, msnp, mail):
        # note: group chat not supported by emesenelib signals design :S
        tmp = self.controller.conversationManager.getOpenConversation(mail)
        if tmp:
            conversationWindow, conversation = tmp
        else:
            return
        
        x, y = conversationWindow.get_position()
        for i in range(50):
            if i%2 == 0:
                conversationWindow.move(x+self.alpha, y)
                conversationWindow.show()
                conversationWindow.move(x, y+self.alpha)
                conversationWindow.show()
            else:
                conversationWindow.move(x-(self.alpha*2), y)
                conversationWindow.show()
                conversationWindow.move(x, y-(self.alpha*2))
                conversationWindow.show()

            gtk.main_iteration(False)
                
        conversationWindow.move(x, y)
        conversationWindow.show()



# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

from core import *

__all__ = [ 'core', 'Switchboard' ]



# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import os
import gc
import sha
import gtk
import ImageAreaSelector

class Avatar( object ):
    '''this class represent an avatar'''

    def __init__( self, path, avatarsPath, dontCache=False,
                  height=96, width=96, thumbHeight=48, thumbWidth=48, 
                  resizeDialog=False ):
        if path == '':
            self.noAvatar()
        elif not self.loadExisting(path):
            self.loadNew(path, avatarsPath, height, width, thumbHeight, 
                thumbWidth, resizeDialog)

    def loadNew( self, path, avatarsPath, height, width, thumbHeight, 
                 thumbWidth, resizeDialog ):
        self.height = height
        self.width = width
        self.thumbHeight = thumbHeight
        self.thumbWidth = thumbWidth

        pixbuf = gtk.gdk.pixbuf_new_from_file( path )
        if resizeDialog:
            areaSelector = ImageAreaSelector.ImageAreaSelectorDialog(pixbuf)
            response, pixbuf = areaSelector.run()
            if response == gtk.RESPONSE_CANCEL:
                self.noAvatar()
                return

        self.image = self.scale( pixbuf, width, height ) 
        self.thumb = self.scale( pixbuf, thumbWidth, thumbHeight )

        imagePath = avatarsPath + os.sep + 'temp.png'
        thumbPath = avatarsPath + os.sep + 'temp_thumb.png'
        self.image.save( imagePath, 'png' )
        self.thumb.save( thumbPath, 'png' )
        
        f = file(imagePath, 'rb')
        hash = sha.new(f.read())
        f.close()
        self.imagePath = avatarsPath + os.sep + hash.hexdigest() + '.png'
        self.thumbPath = avatarsPath + os.sep + hash.hexdigest() + '_thumb.png'
        
        if not os.path.exists( self.thumbPath ):
            os.rename( thumbPath, self.thumbPath )
        else:
            os.remove( thumbPath )
            
        if not os.path.exists( self.imagePath ):
            os.rename( imagePath, self.imagePath )
        else:
            os.remove( imagePath )
            
        # TODO: instead of removing the old we could just not create the new ones..        
        if os.path.dirname( os.path.abspath( path ) ) == os.path.abspath( avatarsPath ):
            os.remove( path )
            
            thumbName = path.split( '.png' )[ 0 ] + '_thumb.png'
            
            try:
                os.remove( thumbName )
            except:
                print 'could not remove ' + thumbName

    def noAvatar(self):
        self.imagePath = ''
        self.thumbPath = ''       

        self.image = None 
        self.thumb = None

        self.height = 0
        self.width = 0
        self.thumbHeight = 0
        self.thumbWidth = 0

    def loadExisting( self, path ):
        self.imagePath = path
        self.thumbPath = path[:-4] + '_thumb.png'       

        try:
            self.image = gtk.gdk.pixbuf_new_from_file( self.imagePath ) 
            self.thumb = gtk.gdk.pixbuf_new_from_file( self.thumbPath ) 
        except:
            return False

        self.height = self.image.get_height()
        self.width = self.image.get_width()
        self.thumbHeight = self.thumb.get_height()
        self.thumbWidth = self.thumb.get_width()
        return True
            
    def getImagePath( self ):
        return self.imagePath
    
    def getThumbPath( self ):
        return self.thumbPath
    
    def getImage( self ):
        return self.image
    
    def getThumb( self ):
        return self.thumb

    def scale( self, image, width, height ):
        h,w = image.get_height(), image.get_width()
        width_max, height_max = width, height
        width=float( image.get_width() )
        height=float( image.get_height() )
        if ( width/width_max ) > ( height/height_max ):
            height=int( ( height/width )*width_max )
            width=width_max
        else:
            width=int( ( width/height )*height_max )
            height=height_max

        image = image.scale_simple( width, height, gtk.gdk.INTERP_BILINEAR )
        gc.collect() # Tell Python to clean up the memory
        return image
